/* Forth machine based on SOD32
*/

#ifdef WIN32
	#include <winsock.h>
#define SHUT_WR	SD_SEND
#else
	#include <termios.h>
	#include <sys/socket.h>
	#include <sys/ioctl.h>
	#include <signal.h>
	#define closesocket close
	#define ioctlsocket ioctl
#endif

#include <stdio.h>
#include <signal.h>
#include <unistd.h>
#include <errno.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <pthread.h>

#include <sys/types.h>
#include <stdlib.h>
#include <dirent.h>

#include "rsa.h"


#define UNS8  unsigned char
#define UNS16 unsigned short
#define INT32 int
#define UNS32 unsigned int
#define INT64 long long
#define UNS64 unsigned long long

#define MEMSIZE (1<<18) /* must be a power of two */
#define MEMMASK MEMSIZE-1 /* mask for addresses to force them into range */

#define MIN(X,Y) ((X) < (Y) ? (X) : (Y))
#define MAX(X,Y) ((X) > (Y) ? (X) : (Y))

extern char * inet_addr_buf;
extern unsigned CIPAddr;
unsigned getmyip();
unsigned hostname2ip(char * hostname);
int BidExec(char * FileN,UNS32 nn);


#define  TIB_SAZE 256

typedef struct {
	UNS32 oSTART;
	UNS32 oSP;
	UNS32 oTO_IN;
	UNS32 oNTIB;
	UNS8  oTTIB[TIB_SAZE];
	UNS32 oATIB;
	UNS32 oSTATE;
	UNS32 oHANDLER;
	UNS32 ooverflow;
	UNS32 oDPOINT;
	UNS32 oLAST;
	UNS32 oVFORTH;
	UNS32 oCONTEXT[17];
	UNS32 oCURRENT;
	UNS32 oOUTPUT;
	UNS32 oINPUT;
}  f_boot_t ;

f_boot_t * f_boot;

#define farrd(reg)  ((int)&(f_boot->reg) - (int)(f_boot))

#define SP		farrd(oSP)
#define TO_IN		farrd(oTO_IN)
#define NTIB	 	farrd(oNTIB)
#define TTIB	 	farrd(oTTIB)
#define ATIB	 	farrd(oATIB)
#define STATE	 	farrd(oSTATE)
#define HANDLER 	farrd(oHANDLER)
#define overflow	farrd(ooverflow)
#define DPOINT		farrd(oDPOINT)
#define LAST		farrd(oLAST)
#define VFORTH		farrd(oVFORTH)
#define CONTEXT		farrd(oCONTEXT)
#define CURRENT		farrd(oCURRENT)
#define OUTPUT		farrd(oOUTPUT)
#define INPUT		farrd(oINPUT)

#define vocs_beg 	sizeof(*f_boot)


#ifdef WIN32

#define WCELL(reg) (*(UNS16*)((UNS32)mem+(reg&MEMMASK)))
#define CELL(reg)  (*(UNS32*)((UNS32)mem+(reg&MEMMASK)))
#define DCELL(reg) (*(UNS64*)((UNS32)mem+(reg&MEMMASK)))
#define FCELL(reg) (*(float*)((UNS32)mem+(reg&MEMMASK)))

#define FSTART	(*(UNS32*)((UNS32)mem))
char * HPATH="HOMEPATH";

#else
#define _O_BINARY 0
#define WCELL(reg) (*(UNS16*)&mem[reg&MEMMASK])
#define CELL(reg)  (*(UNS32*)&mem[reg&MEMMASK])
#define DCELL(reg) (*(UNS64*)&mem[reg&MEMMASK])
#define FCELL(reg) (*(float*)&mem[reg&MEMMASK])

#define FSTART	(*(UNS32*)&mem[0])
char * HPATH="HOME";

#endif

#define HERE	CELL(DPOINT)

#define last	CELL(LAST)
#define context	CELL(CELL(CONTEXT))
#define current	CELL(CELL(CURRENT))
#define TIB	CELL(ATIB)


/* Key for the thread-specific buffer */
 static pthread_key_t buffer_key;

 /* Once-only initialisation of the key */
 static pthread_once_t buffer_key_once = PTHREAD_ONCE_INIT;

#if 0
#if 1
#define mem ((UNS8 *) pthread_getspecific(buffer_key))
#define sp (*(UNS32*)&mem[SP])
#else
volatile register UNS32 sp;
volatile register UNS8 * mem;
#endif
#endif

#define MEMDEF register UNS8 * mem = ((UNS8 *) pthread_getspecific(buffer_key));
#define sp (*(UNS32*)&mem[SP])


UNS32 save_ip,interrupt;
volatile UNS32 stage=0;
volatile UNS8 * mainmem=NULL;
volatile UNS8 * pmainmem;
UNS32 accdat;
UNS32 HostsDat;
UNS32 CountHostPos;
char HomePath[156];
char * pabdir ="/FPPublic0";
char * pabdir1="/FPPublic1";
char * pabdir2="/FPPublic2";
char * pabdir3="/FPPublic3";
char * hiddir ="/.FPauk0";
char * hiddir1="/.FPauk1";
char * hiddir2="/.FPauk2";
char * hiddir3="/.FPauk3";
UNS32 NPort;

UNS64 helpTime;
UNS64 wwwTime;
UNS32 helpIPAddr;

#define LoginSize 32

typedef struct {
	UNS8 ipaddr[66];
	UNS16 port;
	UNS32 number;
	UNS32 pass;
	UNS32 passo;
}  f_hosts_t ;

f_hosts_t f_hosts;

typedef struct { // of client file
 UNS32 pass;
 UNS16 state;
 UNS16 pas2;
 UNS64 account;
 UNS64 barter;
 UNS32 barterN;

}  f_data_t ;


typedef struct {
	UNS32 netid;
	UNS32 fpos;
	UNS8 busy;
	UNS8 pp;
	UNS64 accountSpent;
	UNS64 accountRecd;
	UNS64 barterSpent;
	UNS32 CIPAddr;
	f_data_t data;
}  f_struct_t ;

#define f_structMax  (1<<9)-1

f_struct_t f_struct[f_structMax+1];
pthread_mutex_t mutexacc;

//UNS32 netid;

#define BYTE(reg) mem[reg&MEMMASK]

#define FileFDMax (1<<5)-1
#define fpoutput  fileids[CELL(OUTPUT)&FileFDMax]
#define CLIP addr&=MEMMASK;len&=MEMMASK;if(addr+len>MEMSIZE)len=MEMSIZE-addr;
#define QFERR if(!fp)  {ior=-201;CELL(sp)=ior;break;}
#define FILEID(n) fp=fileids[n&FileFDMax];


//#define PUSH(n)	(sp-=4,CELL(sp)=n)
//#define POP()	(CELL(sp),sp+=4)


void inthandler(int s)
{
 signal(SIGINT,inthandler);
 interrupt=12;
}

void alarmhandler(int s)
{
 interrupt=16;
}

#ifndef WIN32

char key_query_char = 0;


int getch(void)
{
	char ch;
	int n;
	struct termios t1, t2;

	tcgetattr(0, &t1);
	
	 t2 = t1;
	 t2.c_lflag &= ~ICANON;
	 t2.c_lflag &= ~ECHO;
	 t2.c_lflag |= ISIG;
	 t2.c_cc[VMIN] = 1;
	 t2.c_cc[VTIME] = 0;
	 tcsetattr(0, TCSANOW, &t2);

	 n = read(0, &ch, 1);

	tcsetattr(0, TCSANOW, &t1);

	return ch;
}

int kbhit() {
	struct termios t1, t2;
	int bytesWaiting;

    tcgetattr(0, &t1);

	t2 = t1;
	t2.c_lflag &= ~ICANON;

    tcsetattr(0, TCSANOW, &t2);

	ioctl(0, FIONREAD, &bytesWaiting);

    tcsetattr(0, TCSANOW, &t1);

    return bytesWaiting;
}


void putch(int c)
{
	char k=c;
	while(write(1,&k,1)<0);
}


SIGINTTypt()
{
	printf("SIGALRM\n");
}


SIGALRMTypt()
{
	printf("SIGALRM\n");
}

SIGSIGPIPETypt()
{
	printf("disconnect\n");
}

SIGSIGTERMTypt()
{
	printf("SIGSIGTERM\n");
}
SIGSIGSEGVTypt()
{
	printf("SIGSIGSEGV\n");
}


#endif

UNS64 TimeCounter()
{ struct timeval tv;
  struct timezone tz;
	UNS64 tt ;
  gettimeofday(&tv, &tz);
  tt=tv.tv_sec;
  return tt*1000000+tv.tv_usec;
}

int make_name(char *filename,char *addr,UNS32 len)
{
 int i;
 for(i=0;i<len && i<256;i++) {
    if(addr[i]=='.' && addr[i+1]=='.') return 0;
    filename[i]=addr[i];
/* Prohibit filenames in other directories */
 }
 filename[i]='\0';
 return 1;
}


load_image(char *name)
{
 UNS32 len;
 FILE *infile;
 MEMDEF
 if((infile=fopen(name,"rb"))==NULL) {
  fprintf(stderr,"Cannot open image file.\n");
  exit(2);
 }
 len=fread(mem,1,MEMSIZE,infile);
}


/* Multiply 32-bit unsigned numbers *a and *b.
   High half of 64-bit result in *a, low half in *b
*/
static void umul(UNS32 *a,UNS32 *b)
{
 UNS32 ah,al,bh,bl,ph,pm1,pm2,pl;
 ah=*a>>16;al=*a&0xffff;
 bh=*b>>16;bl=*b&0xffff;
 pl=al*bl;
 if((ah|bh)==0) {
  ph=0;
 } else {
  pm1=al*bh;
  pm2=ah*bl;
  ph=ah*bh;
  pl=pl+(pm1<<16);
  ph+=(pl<(pm1<<16));
  pl=pl+(pm2<<16);
  ph+=(pl<(pm2<<16));
  ph=ph+(pm1>>16)+(pm2>>16);
 }
 *a=ph;*b=pl;
}

/* Divide 64-bit unsigned number (high half *b, low half *c) by
   32-bit unsigend number in *a. Quotient in *b, remainder in *c.
*/
static void udiv(UNS32 *a,UNS32 *b,UNS32 *c)
{
 UNS32 d,qh,ql;
 int i,cy;
 qh=*b;ql=*c;d=*a;
 if(qh==0) {
  *b=ql/d;
  *c=ql%d;
 } else {
  for(i=0;i<32;i++) {
   cy=qh&0x80000000;
   qh<<=1;
   if(ql&0x80000000)qh++;
   ql<<=1;
   if(qh>=d||cy) {
    qh-=d;
    ql++;
    cy=0;
   }
   *c=qh;
   *b=ql;
  }
 }
}

enum {
sw_nop,
sw_swap,
sw_rot,
sw_0equ,
sw_0Nequ,
sw_0LT,
sw_0GT,
sw_equ,
sw_nequ,
sw_negate,
sw_invert,
sw_ummul,
sw_Cload,
sw_Wload,
sw_load,
sw_1add,
sw_2add,
sw_1sub,
sw_2sub,
sw_2mul,
sw_2div,
sw_add,
sw_sub,
sw_mul,
sw_div,
sw_mod,
sw_celldiv,
sw_cells,
sw_and,
sw_or,
sw_xor,
sw_uLT,
sw_LT,
sw_uGT,
sw_GT,
sw_umin,
sw_umax,
sw_min,
sw_max,
sw_lshift,
sw_rshift,
sw_umSmod,
sw_addcy,
sw_Dadd,
sw_Dsub,
sw_UDmul,
sw_UDdiv,
sw_UDLT,
sw_drop,
sw_2drop,
sw_on,
sw_0ST,
sw_ST,
sw_addST,
sw_cST,
sw_wST,
sw_2ST,
sw_2load,
sw_2swap,
sw_cSTa,
sw_qdup,
sw_dup,
sw_2dup,
sw_over,
sw_rload,
sw_rGT,
sw_GTr,
sw_ip,
sw_lit,
sw_litcomp,
sw_comp,
sw_Wcomp,
sw_Ccomp,
sw_allot,
sw_execute,
sw_compile,
sw_immediate,
sw_FromName,
sw_ret,
sw_femd,
sw_spload,
sw_spST,
sw_rpload,
sw_rpST,
sw_to_catch,
sw_from_catch,
sw_throw,
sw_setalarm,
sw_bye,
sw_branch,
sw_0branch,
sw_BEGIN,
sw_AGAIN,
sw_UNTIL,
sw_AHEAD,
sw_IF,
sw_WHILE,
sw_THEN,
sw_ELSE,
sw_REPEAT,
sw_key,
sw_emit,
sw_keyq,
sw_type,
sw_hdot,
sw_decdot,
sw_count,
sw_str,
sw_StrComp,
sw_Compare,
sw_NextWord,
sw_Parse,
sw_SQ,
sw_PQ,
sw_Source,
sw_SourceST,
sw_SFind,
sw_SearchWordList,
sw_SHeader,
sw_chdir,
sw_dir,
sw_openfile,
sw_closefile,
sw_writeline,
sw_readline,
sw_writefile,
sw_readfile,
sw_WatchdogADD,
sw_typefilet,
sw_help,
sw_deletefile,
sw_repositionfile,
sw_fileposition,
sw_system,
sw_hiddir,
sw_filesize,
sw_restore,
sw_telid,
sw_ipadr,
sw_connect,
sw_tellen,
sw_telp,
sw_setpabdir,
sw_ms,
sw_TimeCounter,
sw_modexp,
sw_gcd,
sw_modinv,
sw_ToSimple,
sw_fnumber,
sw_fadd,
sw_fsub,
sw_fmul,
sw_fdiv,
sw_f2ud,
sw_u2f,
sw_fprint,
sw_fcells,
sw_Stage,
sw_StageAdd,
sw_getmyip,
sw_hostname2ip,
sw_getacc,
sw_getbarter,
sw_FBiteCost,
sw_render,
sw_frombarter,
sw_GetAccKey,
sw_kaccount,
sw_AccountNew,
sw_cnumber,
sw_passadd,
sw_hostpassadd,
sw_OldAcc,
sw_NOldAcc,
sw_NumConnect,
sw_GETOLDACC,
sw_NPort,
sw_barterAdd,
sw_s2cln,
sw_accrual,
sw_hostadd,
sw_bid,
sw_ztstp,
sw_hi
};


void COM(UNS32 dd)
{ MEMDEF
 CELL(HERE)=dd;
HERE+=4;
}

void WCOM(UNS16 dd)
{ MEMDEF
	WCELL(HERE)=dd;
	HERE+=2;
}

void CCOM(UNS8 dd)
{ MEMDEF
 BYTE(HERE)=dd;
  HERE+=1;
}

void StrComp(UNS8 * word,UNS32 size)
{ UNS32 addr1,addr0;
  MEMDEF
addr0=HERE;
addr1=addr0+1;
while(size--)
	   BYTE(addr1++)=*word++;
BYTE(addr0)=addr1-addr0-1;
HERE=addr1;
if(HERE&1)CCOM(0);
}

void SHEADER(UNS8 * word,UNS32 size)
{       MEMDEF
	if(current) WCOM(current-HERE);
	else	WCOM(0);
   last=HERE;
   StrComp(word,size);
}

void HEADER(UNS8 * word)
{	MEMDEF
	size_t i;
    for (i = 0; word[i] != 0; i ++);
	SHEADER(word,i);
}

void IMMED()
{	MEMDEF
	BYTE(last)=BYTE(last)|0x80;
}

void PRIM(UNS8 * word,UNS16 cod)
{	MEMDEF
	HEADER(word);
	current=last;
	WCOM(cod<<1|0x8001);
}

void CONSTANT(char * word,UNS32 dd)
{	MEMDEF
	HEADER(word);
	current=last;
	WCOM(sw_lit<<1|0x4001);
	COM(dd);
}

void CREATE(char * word)
{	MEMDEF
	HEADER(word);
	current=last;
	WCOM(sw_ip<<1|0x4001);
}

void VARIABLE(char * word)
{	MEMDEF
	CREATE(word);
	COM(0);
}

#ifndef memcasecmp
memcasecmp (const void *vs1, const void *vs2, size_t n)
{
  unsigned int i;
  unsigned char const *s1 = (unsigned char const *) vs1;
  unsigned char const *s2 = (unsigned char const *) vs2;
  for (i = 0; i < n; i++)
    {
	unsigned char u1 = *s1++;
	unsigned char u2 = *s2++;
	if (toupper (u1) != toupper (u2))
	return toupper (u1) - toupper (u2);
    }
  return 0;
}
#endif

UNS32 SEARCH(UNS32 wid,  char * word)
{ UNS32 addr=wid;
  MEMDEF
  for(;;)
  { UNS16 link;
//	if( !memcmp(word,&BYTE(addr+1),BYTE(addr)&0x7F) )
	if( !memcasecmp(word,&BYTE(addr+1),BYTE(addr)&0x7F) )
	return addr;
	addr-=2;
	link=WCELL(addr);
	if(!link) return 0;
	addr+=(link^0x8000)-0x8000;
  }
}

UNS32 FromName(UNS32 addr)
{ MEMDEF
 addr+=(BYTE(addr)&0x7f)+1;
  if(addr&1)addr++;
  return addr;
}

void Compile(UNS32 addr)
{	UNS32 cod;
	MEMDEF
	  cod=WCELL(addr);
	  if((cod&0x8001)==0x8001)
		  {WCOM(cod&0x3FFF); return;}
	  WCOM(addr-(HERE+2));

}
void CC(char * word)
{	MEMDEF
	UNS32 addr=current;
	UNS8 size;

	for (size = 0; word[size] != 0; size ++);

	do	{
		addr=SEARCH(addr,word);
	   	if((BYTE(addr)&0x3f)==size ) break;
		addr-=2;
		addr+=(WCELL(addr)^0x8000)-0x8000;
	}while(addr);

	  if(!addr)
	    for(;;);
	  Compile(FromName(addr));
}

void STR(char * str)
{	MEMDEF
	char * str0=str;
	UNS32 addr0;
	WCOM(sw_str<<1|1);
	addr0=HERE;
	CCOM(0);
	while(*str)  CCOM(*str++);
        BYTE(addr0)=(UNS8)(str-str0);
	if(HERE&1)CCOM(0);
}

void TP(char * str)
{ MEMDEF
 STR(str);
WCOM(sw_type<<1|1);
}

void F_END()
{	MEMDEF
	WCOM(0x8001);
	current=last;
}

void BEGIN()
{	MEMDEF
	sp-=4;CELL(sp)=HERE;
}

void AGAIN()
{	MEMDEF
	WCOM(sw_branch<<1|1);
	WCOM(CELL(sp)-HERE);
	sp+=4;
}

void UNTIL()
{	MEMDEF
	WCOM(sw_0branch<<1|1);
	WCOM(CELL(sp)-HERE);
	sp+=4;
}

void AHEAD()
{	MEMDEF
	sp-=4;CELL(sp)=0;
	AGAIN();
	sp-=4;CELL(sp)=HERE-2;
}

void IF()
{	MEMDEF
	sp-=4;CELL(sp)=0;
	UNTIL();
	sp-=4;CELL(sp)=HERE-2;
}

void WHILE()
{ MEMDEF
  UNS32 t;
	IF();
	t=CELL(sp);
	CELL(sp)=CELL(sp+4);
	CELL(sp+4)=t;
}

void THEN()
{ MEMDEF
	WCELL(CELL(sp))=HERE-CELL(sp);
	sp+=4;
}

void ELSE()
{ MEMDEF
  UNS32 t;
	AHEAD();
	t=CELL(sp);
	CELL(sp)=CELL(sp+4);
	CELL(sp+4)=t;
	THEN();
}

void REPEAT()
{	MEMDEF
	AGAIN();
	THEN();
}

void LIT(UNS32 dd)
{	MEMDEF
	WCOM(sw_lit<<1|1);
	COM(dd);
}

void NextWord()
{ MEMDEF
  UNS32 addr,Waddr,Eaddr;
addr=CELL(TO_IN) + TIB ;
Eaddr=TIB+CELL(NTIB);
sp-=8;
	while ( (BYTE(addr) <= ' ') & (addr<Eaddr ) )addr++;
	CELL(sp+4)=Waddr=addr;
	while ( (BYTE(addr++)  > ' ') & (addr<Eaddr )	);
	CELL(sp)=addr-Waddr-1;
	CELL(TO_IN)=addr - TIB;
}

void Parse()
{ MEMDEF
  UNS32 addr,Waddr,Eaddr;
  UNS8 ch;
addr=CELL(TO_IN) + TIB ;
Eaddr=TIB+CELL(NTIB);
ch=BYTE(sp);
sp-=4;
	CELL(sp+4)=Waddr=addr;
	while ( (BYTE(addr++)  != ch) & (addr<Eaddr )	);
	CELL(sp)=addr-Waddr-1;
	CELL(TO_IN)=addr - TIB;
}

void SQ()
{	MEMDEF
	sp-=4; CELL(sp)=(UNS32)'"';Parse();
	if(CELL(STATE))
	{		UNS32 addr,size;
	WCOM(sw_str<<1|1);
	addr=CELL(sp+4);
	size=CELL(sp);
	size=MIN(size, MEMSIZE-addr);
	StrComp(&BYTE(addr),size);
  sp+=8;
	}
}
void Source()
{	MEMDEF
	sp-=8;
	CELL(sp+4)=TIB;
	CELL(sp)=CELL(NTIB);
}

void SourceST()
{	MEMDEF
	UNS32 len,addr;
	CELL(NTIB)=len=CELL(sp);
	addr=CELL(sp+4);
	while(len--)	BYTE(TIB+len)=BYTE(addr+len);
	sp+=8;
}

void SearchWordList() // ( c-addr u wid --- 0 | xt 1 xt -1 )
{	MEMDEF
  UNS32 addr=CELL(CELL(sp));
  UNS8  size=CELL(sp+4);
  char * word=&BYTE(CELL(sp+8));
  for(;;)
    {  	addr=SEARCH(addr,word);
	if(!addr)
	{
		sp+=8; CELL(sp)=0; return;
	}
	if((BYTE(addr)&0x3f)==size )
	{ sp+=4;
	  if(BYTE(addr)&0x80)
	   CELL(sp)=1;
	  else
	   CELL(sp)=-1;
	  CELL(sp+4)=FromName(addr);
	  return;
	}
		addr-=2;
		addr+=(WCELL(addr)^0x8000)-0x8000;
    }

}

void SFind()
{	int voc=CONTEXT;
	MEMDEF
	sp-=4; CELL(sp)=0;
	while( CELL(voc) )
	{	sp+=4;
	    sp-=4;CELL(sp)=CELL(sp+8);
	    sp-=4;CELL(sp)=CELL(sp+8);
		sp-=4;CELL(sp)=CELL(voc);
		SearchWordList();
		if(CELL(sp))
		{
			CELL(sp+8)=CELL(sp);
			CELL(sp+12)=CELL(sp+4);
			sp+=8;
			return;
		}	 voc+=4;
	}
}

ssize_t twrite (int __fd, __const void *__buf, size_t __n)
{	MEMDEF
	if(__fd&0x80000000) return(send(__fd^0x80000000,__buf,__n,0));
	else	return(write(__fd,__buf,__n));
}

ssize_t tread (int __fd, __const void *__buf, size_t __n)
{ int nn;
	if(__fd&0x80000000)	return(recv(__fd^0x80000000,__buf,__n,0));
	else	return(read(__fd,__buf,__n));
}

#ifndef eof
int eof(int __fd)
{ int fpt,flg;
	fpt=lseek(__fd, (off_t) 0, SEEK_CUR);
	flg=(fpt==lseek(__fd, (off_t) 0, SEEK_END));
	lseek(__fd, (off_t) fpt, SEEK_SET);
	return flg;
 }
#endif

UNS32 LoginSearch(UNS32 ipaddr,UNS32 port)
{
//	lseek(HostsDat, (off_t) 0, SEEK_SET);
	CountHostPos=lseek(HostsDat, (off_t) 0, SEEK_END);
	while(CountHostPos)
	{
		CountHostPos=lseek(HostsDat, CountHostPos-sizeof(f_hosts_t), SEEK_SET);
		read(HostsDat, (char *) &f_hosts , sizeof(f_hosts_t));
		printf("p=%d %d ip= %d %d\n",f_hosts.port,port,ipaddr,hostname2ip(f_hosts.ipaddr) );
		if(f_hosts.port==port)
		if(ipaddr==hostname2ip(f_hosts.ipaddr))
		{ return 1;}
	
//	}while(!eof(HostsDat));
	};
	return 0;
}

void accrual( UNS32 fpos, UNS64 account)
{	UNS32 ii=0; 
 	 	 	
 pthread_mutex_lock(&mutexacc);

	if(fpos>=lseek(accdat, (off_t) 0, SEEK_END))goto accrual_end;
	 lseek(accdat, (off_t) fpos, SEEK_SET);
	 for(;ii<f_structMax;ii++)
	 {
		if(f_struct[ii].fpos==fpos)
		{
		f_struct[ii].accountRecd+=account;
		f_struct[ii].data.account=f_struct[ii].accountRecd-f_struct[ii].accountSpent;
		write(accdat, (char *) &f_struct[ii].data , sizeof(f_data_t));
		goto accrual_end;
		}
	}
	{	f_data_t data;
		read(accdat, (char *) &data , sizeof(f_data_t));
		data.account+=account;
	 	lseek(accdat, (off_t) fpos, SEEK_SET);
		write(accdat, (char *) &data , sizeof(f_data_t));
	}

accrual_end:	pthread_mutex_unlock(&mutexacc);
}

void accrualn( UNS32 nn, UNS64 account)
{
 accrual( nn*sizeof(f_data_t), account);
}

int drawaln( UNS32 nn, UNS64 account)
{	UNS32 ii=0; int res=-1;
    UNS32 fpos=nn*sizeof(f_data_t);

 pthread_mutex_lock(&mutexacc);

	if(fpos>=lseek(accdat, (off_t) 0, SEEK_END))goto accrual_end;
	 lseek(accdat, (off_t) fpos, SEEK_SET);
	 for(;ii<f_structMax;ii++)
	 {
		if(f_struct[ii].fpos==fpos)
		{
			  if( account > f_struct[ii].accountRecd-f_struct[ii].accountSpent)
				  goto accrual_end;

			  f_struct[ii].accountRecd-=account;
			  res=0;
		goto accrual_end;
		}
	}
	{	f_data_t data;
		read(accdat, (char *) &data , sizeof(f_data_t));
		  if( account > data.account)
			  goto accrual_end;
		  res=0;
		data.account-=account;
	 	lseek(accdat, (off_t) fpos, SEEK_SET);
		write(accdat, (char *) &data , sizeof(f_data_t));
	}

accrual_end:	pthread_mutex_unlock(&mutexacc);
 return res;
}


accSave(UNS32  t_ii)
{
	if( f_struct[t_ii].accountSpent | f_struct[t_ii].barterSpent )
	{
		pthread_mutex_lock(&mutexacc);

		lseek(accdat, f_struct[t_ii].fpos, SEEK_SET);

		f_struct[t_ii].data.account=f_struct[t_ii].accountRecd-f_struct[t_ii].accountSpent;

		f_struct[t_ii].data.barter-=f_struct[t_ii].barterSpent;
		f_struct[t_ii].barterSpent=0;

		write(accdat, (char *) &f_struct[t_ii].data , sizeof(f_data_t));

		pthread_mutex_unlock(&mutexacc);
	}
}

int posis(fpos)
{ int ii=0;
	for(;ii<f_structMax;ii++)
	{
		if(f_struct[ii].fpos==fpos)
{  printf("posis=%d,%d\n",ii,fpos);
  return 1;
}
	}
	 return 0;
}

UNS64 FBiteCost=100l;

int  WatchdogN= 70000;

void virtual_machine(UNS32 t_ii)
{
	MEMDEF
 register rp,ret,ip,ireg,t;

 UNS32 addr,len,ior=0,ips,CCNN=-1 , res;
 UNS32 zzz=0;
 UNS32 TELID=-1;
 UNS64 dtmp;
 UNS32 backup_mode=0;
 char filename[256];
 char * fpath0="./";
 char * fpath1="";
 char * fpath2="";
 char	tplast='\n';
 int	calltelp=0;
 int	telpCount=0;
 int	lstage=stage;
 UNS32 AccKey;
 UNS32 Watchdog=WatchdogN;
 UNS32 netid;
 netid=f_struct[t_ii].netid;
 if(f_struct[t_ii].CIPAddr==0x100007f) lstage=4;


 UNS32 fileids[FileFDMax+1]={
		0,1,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,
		0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,
		0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,
		0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0, };
 int fp;

 printf("virtual_machine\n");
 if(netid)
 {	fileids[0]=netid;
	fileids[1]=netid;
 }
 if(lstage>10)
 {	fpath0=HomePath;
	fpath1=pabdir;
 }

 interrupt=0;
 rp=MEMSIZE;
 ip=FSTART;

 for(;;) {

	if( (lstage>10) & (Watchdog==0))
	{
		twrite(fpoutput,"Watchdog reset\n",15);

		interrupt=0;
		rp=MEMSIZE;
		ip=FSTART;
		memcpy(mem,mainmem,MEMSIZE);

		Watchdog=WatchdogN;
	}
	Watchdog--;
  if(interrupt) {
   ireg=1;
  doint:
   rp-=4;
  CELL(rp)=ip;
   sp-=4;CELL(sp)=ireg;
   ip=CELL(interrupt);
   interrupt=0;
  }
  ireg=WCELL(ip);
  ips=ip;
  ip+=2;
  ip&=MEMMASK;
  //if(netid)  {   twrite(netid, "hello " ,6);  }
  ExecL: /* Instruction restart after interrupt */
  if(!(ireg&1)) {
	rp-=4;
	CELL(rp)=ip;
	ip+= (ireg^0x8000) - 0x8000;
  } else {
    switch( (ireg&0x3fff)>>1) {
  case sw_swap:	/*swap*/ t=CELL(sp);CELL(sp)=CELL(sp+4); CELL(sp+4)=t;
  case sw_nop:	/*nop*/	 break;
  case sw_rot:	/*rot*/  t=CELL(sp+8);CELL(sp+8)=CELL(sp+4);
				CELL(sp+4)=CELL(sp);CELL(sp)=t;break;
  case sw_0equ:	/*0=*/	CELL(sp)=-!CELL(sp);break;
  case sw_0Nequ:	/*0<>*/   CELL(sp)=-!!CELL(sp);break;
  case sw_0LT:	/*0<*/   CELL(sp)=-((INT32)CELL(sp)<0);break;
  case sw_0GT:	/*0>*/   CELL(sp)=-((INT32)CELL(sp)>0);break;
  case sw_equ:	/*=*/  	 CELL(sp+4)=-(CELL(sp)==CELL(sp+4));sp+=4;break;
  case sw_nequ:	/*=*/  	 CELL(sp+4)=-(CELL(sp)!=CELL(sp+4));sp+=4;break;
  case sw_negate:	/*negate*/	CELL(sp)=-CELL(sp);break;
  case sw_invert:	/*invert*/	CELL(sp)=~CELL(sp);break;
  case sw_ummul:	/* um* */	umul(&CELL(sp),&CELL(sp+4));break;
  case sw_Cload:	/* C@ */	CELL(sp)=BYTE(CELL(sp));break;
  case sw_Wload:	/* W@ */	CELL(sp)=WCELL(CELL(sp));break;
  case sw_load:	/* @ */		CELL(sp)=CELL(CELL(sp));break;
  case sw_1add:	/* 1+ */	CELL(sp)++;break;
  case sw_2add:	/* 2+ */	CELL(sp)+=2;break;
  case sw_1sub:	/* 1- */	CELL(sp)--;break;
  case sw_2sub:	/* 2+ */	CELL(sp)-=2;break;
  case sw_2mul:	/* 2* */	CELL(sp)*=2;break;
  case sw_2div:	/* 2/ */	CELL(sp)=(INT32)CELL(sp)/2;break;
  case sw_add:	/* + */		CELL(sp+4)=CELL(sp)+CELL(sp+4);sp+=4;break;
  case sw_sub:	/* - */		CELL(sp+4)-=CELL(sp);sp+=4;break;
  case sw_mul:	/* * */CELL(sp+4)=(INT32)CELL(sp)*(INT32)CELL(sp+4);sp+=4;break;
  case sw_div:	/* / */   if(CELL(sp))
			{ CELL(sp+4)=(INT32)CELL(sp+4)/(INT32)CELL(sp);sp+=4;break;
			} CELL(sp+4)=-1; sp+=4;break;
  case sw_mod:	/* mod */
	if(CELL(sp))
	{ CELL(sp+4)=(INT32)CELL(sp+4)%(INT32)CELL(sp);sp+=4;break;
	} CELL(sp+4)=0; sp+=4;break;
  case sw_celldiv:	/* cell/ */ CELL(sp)>>=2;break;
  case sw_cells:	/* cells */ CELL(sp)<<=2;break;
  case sw_and:	/* and */ CELL(sp+4)=CELL(sp)&CELL(sp+4);sp+=4;break;
  case sw_or:	/* or  */ CELL(sp+4)=CELL(sp)|CELL(sp+4);sp+=4;break;
  case sw_xor:	/* xor */ CELL(sp+4)=CELL(sp)^CELL(sp+4);sp+=4;break;
  case sw_uLT:	/* u<  */ CELL(sp+4)=-(CELL(sp+4)<CELL(sp));sp+=4;break;
  case sw_LT:	/* <   */ CELL(sp+4)=-((INT32)CELL(sp+4)<(INT32)CELL(sp));sp+=4;break;
  case sw_uGT:	/* u>  */ CELL(sp+4)=-(CELL(sp+4)>CELL(sp));sp+=4;break;
  case sw_GT:	/* >  */ CELL(sp+4)=-((INT32)CELL(sp+4)>(INT32)CELL(sp));sp+=4;break;
  case sw_umin:	/* umin */ CELL(sp+4)=MIN(CELL(sp+4),CELL(sp));sp+=4;break;
  case sw_umax:	/* umax */ CELL(sp+4)=MAX(CELL(sp+4),CELL(sp));sp+=4;break;
  case sw_min:	/* min */ CELL(sp+4)=MIN((INT32)CELL(sp+4),(INT32)CELL(sp));sp+=4;break;
  case sw_max:	/* max */ CELL(sp+4)=MAX((INT32)CELL(sp+4),(INT32)CELL(sp));sp+=4;break;
  case sw_lshift:/*lshift*/CELL(sp+4)<<=CELL(sp);sp+=4;break;
  case sw_rshift:/*rshift*/CELL(sp+4)>>=CELL(sp);sp+=4;break;
  case sw_umSmod:/*um/mod*/
	if(CELL(sp)<=CELL(sp+4))
	{ /*overflow */
//	  interrupt=overflow;  goto doint;
	  sp+=4;CELL(sp)=-1;CELL(sp+4)=-1; break;
	}
	udiv(&CELL(sp),&CELL(sp+4),&CELL(sp+8));
	sp+=4;	break;

  case sw_addcy:	/* +cy */
	{ UNS32 sum;
	 if(CELL(sp)) { /* carry in */
		sum=CELL(sp+8)+CELL(sp+4)+1;
		sp+=4;
		CELL(sp)=(sum<=CELL(sp));
		CELL(sp+4)=sum;
	 } else {
		sum=CELL(sp+8)+CELL(sp+4);
		sp+=4;
		CELL(sp)=(sum<CELL(sp));
		CELL(sp+4)=sum;
	 }
        }break;
  case sw_Dadd:	/* d+ */

	dtmp=	((UNS64)CELL(sp+8)<<32) + (UNS64)CELL(sp+12) +
		((UNS64)CELL(sp)<<32) + (UNS64)CELL(sp+4);
		CELL(sp+8)=dtmp>>32;
		CELL(sp+12)=dtmp; sp+=8;break;

  case sw_Dsub:	/* d- */

	dtmp=	(((UNS64)CELL(sp+8)<<32) + (UNS64)CELL(sp+12)) -
		(((UNS64)CELL(sp)<<32) + (UNS64)CELL(sp+4));
		CELL(sp+8)=dtmp>>32;
		CELL(sp+12)=dtmp; sp+=8;break;

  case sw_UDmul:	/* ud* */

	dtmp=	(((UNS64)CELL(sp+8)<<32) + (UNS64)CELL(sp+12)) *
		(((UNS64)CELL(sp)<<32) + (UNS64)CELL(sp+4));
		CELL(sp+8)=dtmp>>32;
		CELL(sp+12)=dtmp; sp+=8;break;

  case sw_UDdiv:	/* ud/ */

	dtmp=	(((UNS64)CELL(sp+8)<<32) + (UNS64)CELL(sp+12)) /
		(((UNS64)CELL(sp)<<32) + (UNS64)CELL(sp+4));
		CELL(sp+8)=dtmp>>32;
		CELL(sp+12)=dtmp; sp+=8;break;

  case sw_UDLT:	/* ud< */

	t=	-((((UNS64)CELL(sp+8)<<32) + (UNS64)CELL(sp+12)) <
		(((UNS64)CELL(sp)<<32) + (UNS64)CELL(sp+4)));
		CELL(sp+12)=t; sp+=12;break;

  case sw_drop:	/* drop */ sp+=4;break;
  case sw_2drop:/* drop	*/ sp+=8;break;
  case sw_on:	/* 0!	*/ CELL(CELL(sp))=-1;sp+=4;break;
  case sw_0ST:	/* 0!	*/ CELL(CELL(sp))=0;sp+=4;break;
  case sw_ST:	/*  !	*/ CELL(CELL(sp))=CELL(sp+4);sp+=8;break;
  case sw_addST:/*  +!	*/ CELL(CELL(sp))+=CELL(sp+4);sp+=8;break;
  case sw_cST:	/* C!	*/ BYTE(CELL(sp))=CELL(sp+4);sp+=8;break;
  case sw_wST:	/* W!	*/ WCELL(CELL(sp))=CELL(sp+4);sp+=8;break;
  case sw_2ST:	/* 2!	*/ DCELL(CELL(sp))=DCELL(sp+4);sp+=12;break;
  case sw_2load:/* 2@	*/ sp-=4; DCELL(sp)=DCELL(CELL(sp+4));break;
  case sw_2swap:/* 2swap */

	dtmp=DCELL(sp);
	DCELL(sp)=DCELL(sp+8);
	DCELL(sp+8)=dtmp;break;

  case sw_cSTa:/* c!a  */

	t=CELL(sp);BYTE(t)=CELL(sp+4);sp+=4;
	CELL(sp)=t;break;

  case sw_qdup:	/* ?dup	*/ if(CELL(sp)==0)break;
  case sw_dup:	/* dup	*/ sp-=4;CELL(sp)=CELL(sp+4);break;
  case sw_2dup:	/* 2dup */ sp-=4;CELL(sp)=CELL(sp+8);
  case sw_over:	/* over */ sp-=4;CELL(sp)=CELL(sp+8);break;
  case sw_rload:/* r@	*/ sp-=4;CELL(sp)=CELL(rp);break;
  case sw_rGT:	/* r>	*/ sp-=4;CELL(sp)=CELL(rp);rp+=4;break;
  case sw_GTr:	/* >r	*/ rp-=4;CELL(rp)=CELL(sp);sp+=4;break;
  case sw_ip:	/* ip	*/ sp-=4;CELL(sp)=ip;ip+=4;break;
  case sw_lit:	/*lit	*/ sp-=4;CELL(sp)=CELL(ip);ip+=4;break;

  case sw_litcomp:/*lit	*/ WCOM(sw_lit<<1|1);
  case sw_comp:	/* ,	*/ COM(CELL(sp));	sp+=4;break;
  case sw_Wcomp:/* W,	*/ WCOM(CELL(sp));	sp+=4;break;
  case sw_Ccomp:/* C,	*/ CCOM(CELL(sp));	sp+=4;break;
  case sw_allot:/* allot */HERE+=CELL(sp);	sp+=4;break;
  case sw_execute:/* execute */	 rp-=4;CELL(rp)=ip; ip=CELL(sp); sp+=4;break;
  case sw_compile:/* compile */	 Compile(CELL(sp));	sp+=4;break;
  case sw_immediate:/* immediate */	IMMED();break;
  case sw_femd:/*return*/	F_END();break;
  case sw_FromName:/*return*/	CELL(sp)=FromName(CELL(sp));break;
  case sw_spload:/* sp@ */ sp-=4;CELL(sp)=sp+4;break;
  case sw_spST:  /* sp! */ sp=CELL(sp);break;
  case sw_rpload:/* rp@ */ sp-=4;CELL(sp)=rp;break;
  case sw_rpST:  /* rp! */ rp=CELL(sp);sp+=4;break;

//  case sw_setalarm: /* setalarm */ setalarm(CELL(sp));sp+=4;break;

  case sw_to_catch:

	rp-=12;
	CELL(rp+8)=CELL(HANDLER);
	CELL(rp+4)=sp;
	CELL(rp)=ip;
	CELL(HANDLER)=rp;
	ip=CELL(sp);
	sp+=4;
	break;

  case sw_from_catch:

	CELL(HANDLER)=CELL(rp+8-4);
	rp+=12-4;
	sp-=4;CELL(sp)=0;
	break;

  case sw_throw:
	if( !CELL(sp)) { sp+=4; break; }
	if( !CELL(HANDLER)) {exit(CELL(sp));break;}
	{ UNS32 savesp=CELL(sp);
	  rp=CELL(HANDLER);
	  CELL(HANDLER)=CELL(rp+8);
	  sp=CELL(rp+4);
	  CELL(sp)=savesp;
	  ip=CELL(rp)+2; // over from_catch
	  rp+=12;
	  break;
	}

  case sw_bye: /*exit*/

	if(netid&0x80000000)
	{ shutdown(netid^0x80000000, SHUT_WR);
	 closesocket(netid^0x80000000) ;
	}
	accSave(t_ii);
	f_struct[t_ii].busy=0;
	f_struct[t_ii].fpos=-1;
	return;

  case sw_branch: /*branch*/	ip += (WCELL(ip)^0x8000) - 0x8000;break;

  case sw_0branch: /*0branch*/

	if(CELL(sp)) ip+=2; else ip+= (WCELL(ip)^0x8000) - 0x8000; sp+=4; break;

  case sw_BEGIN:  BEGIN();break;
  case sw_AGAIN:  AGAIN();break;
  case sw_UNTIL:  UNTIL();break;
  case sw_AHEAD:  AHEAD();break;
  case sw_IF:     IF();break;
  case sw_WHILE:  WHILE();break;
  case sw_THEN:   THEN();break;
  case sw_ELSE:   ELSE();break;
  case sw_REPEAT: REPEAT();break;

  case sw_key: /*read char */ sp-=4;     CELL(sp)= getch();     break;

  case sw_emit: /*print char*/

	twrite(fpoutput,&BYTE(sp),1);
	sp+=4;break;

  case sw_keyq: /*check key */ sp-=4;CELL(sp)=-kbhit();break;

  case sw_type: /*type*/

	if( CELL(sp+4)+CELL(sp) < MEMSIZE )
	twrite(fpoutput,&BYTE(CELL(sp+4)),CELL(sp));
	sp+=8;break;

  case sw_hdot: /*print hex*/
     {	char buffer[44];
	sprintf (buffer, "%X ", CELL(sp));
	twrite(fpoutput,buffer,strlen(buffer));
     }	sp+=4;break;

  case sw_decdot: /*print decimal*/
     {	char buffer[44];
	sprintf (buffer, "%d ", CELL(sp));
	twrite(fpoutput,buffer,strlen(buffer));
     }	sp+=4;break;

  case sw_count: /*count*/ sp-=4;  CELL(sp)= BYTE(CELL(sp+4)); CELL(sp+4)++; break;
  case sw_str: /*(S")*/

	sp-=8; t=BYTE(ip++); CELL(sp)=t; CELL(sp+4)=ip;
	ip+=t; if(ip&1)ip++; break;

  case sw_StrComp: /* S, */

	if( CELL(sp+4)+CELL(sp) < MEMSIZE )	StrComp(&BYTE(CELL(sp+4)),CELL(sp));
	sp+=8;break;

  case sw_Compare: /* Compare */
    { int addr1,len1,addr2,len2;
	len1=CELL(sp);    addr1=CELL(sp+4);
	len2=CELL(sp+8);  addr2=CELL(sp+12); sp+=12;
	if(len1!=len2) { CELL(sp)=len1-len2; break; }
	if( addr1+len1 < MEMSIZE )
	if( addr2+len2 < MEMSIZE )
		CELL(sp)=memcmp(&BYTE(addr1),&BYTE(addr2),len1);
	break;
    }
  case sw_NextWord:	NextWord(); break;
  case sw_Parse:	Parse(); break;
  case sw_SQ: /* S" */ SQ();	break;
  case sw_PQ: /* ." */ SQ();	WCOM(sw_type<<1|1); break;
  case sw_SourceST:		SourceST(); break;
  case sw_Source:		Source(); break;
  case sw_SearchWordList:	SearchWordList(); break;
  case sw_SFind:		SFind(); break;
  case sw_SHeader:

	if( CELL(sp+4)+CELL(sp) < MEMSIZE )
	SHEADER( &BYTE(CELL(sp+4)), CELL(sp) );
     	sp+=8; break;

  case sw_chdir:

	if(lstage>10)	break;

	if( CELL(sp)==0 )
	{	fpath0="./";
		fpath1="";	break;
	}
		fpath0=HomePath;

	if( CELL(sp)==1 )
	{	fpath1=hiddir;	break;
	}

		fpath1=pabdir;	break;

  case sw_dir:
     {	DIR *dir;
	struct dirent *entry;
	char dirname[256];
	char flname[256];
	len=CELL(sp);addr=CELL(sp+4);
	if(!make_name(filename,mem+addr,len)) {
		twrite(fpoutput,".. is unacceptable",18);
		break;}
	strcpy(dirname,fpath0);
	strcat(dirname,fpath1);
	strcat(dirname,"/");
	strcat(dirname,filename);
	strcat(dirname,"/");
	{ char buffer[144];
	  sprintf (buffer,"%s\n",dirname);
	  twrite(fpoutput,buffer,strlen(buffer));
	}
	if ((dir = opendir(dirname)) == NULL)
		twrite(fpoutput,"opendir() error",15);
	else
	{    while ((entry = readdir(dir)) != NULL)
	   {
		struct stat stbuf;
		if(entry->d_name[0]!='.')
		{	strcpy(flname,dirname);
			strcat(flname,entry->d_name);
		 if(stat(flname, &stbuf)==0)
		 {	char buffer[44]; char * ccc;
			ccc=ctime(&stbuf.st_mtime);
			write(1,ccc,strlen(ccc));

			strcpy(buffer, ctime(&stbuf.st_mtime));	buffer[strlen(buffer)-1]=0;
			sprintf(buffer,"%s %d\t",buffer,stbuf.st_size);
			if( (stbuf.st_mode & S_IFMT) == S_IFDIR ) strcat(buffer,"/");
			strcat(buffer,entry->d_name);
			strcat(buffer,"\n");
			twrite(fpoutput,buffer,strlen(buffer));
		 }
		}
	    }
	    closedir(dir);
	  }
     }  break;

  case sw_openfile: /*open-file*/

	sp+=4;len=CELL(sp);addr=CELL(sp+4);
	CLIP if(!make_name(filename,mem+addr,len)) {ior=-202;
	CELL(sp)=ior;break;}
	{ int i;  char flname[256];

		strcpy(flname,fpath0);
		strcat(flname,fpath1);
		strcat(flname,"/");
		strcat(flname,filename);

		for(i=6;i<FileFDMax;i++) { if(!fileids[i])break; }
		if(i==20){ior=-201;CELL(sp)=ior;break;}

		if(lstage<10)	fp=open(flname,CELL(sp-4)|_O_BINARY,S_IRUSR | S_IWUSR );
		else		fp=open(flname,O_RDONLY|_O_BINARY,S_IRUSR | S_IWUSR );
		ior=0;
		if(fp>0) fileids[i]=fp;
		else ior=200;
		CELL(sp+4)=i;
	}	CELL(sp)=ior;	break;

  case sw_closefile: /*close-file*/

	FILEID(CELL(sp)) QFERR close(fp);
	fileids[CELL(sp)&FileFDMax]=0;
	CELL(sp)=ior;break;;

  case sw_writeline: /*write-line*/

	sp+=8;FILEID(CELL(sp-8)) len=CELL(sp-4);
	addr=CELL(sp);CLIP
	twrite(fp,mem+addr,len);
	twrite(fp,"\n",1);    ior=0;
	CELL(sp)=ior;break;

  case sw_readline: /*read-line*/  // ( c-addr u1 fileid -- u2 flag ior )

	FILEID(CELL(sp))
	len=CELL(sp+4)+2;
	addr=CELL(sp+8);CLIP
	res=0;
	if(fp&0x80000000)
		{ // printf("Wd=%d\n",Watchdog);
		   accSave(t_ii);
		}
		else
		{ if(lstage>10)
		  if( len*FBiteCost > f_struct[t_ii].accountRecd-f_struct[t_ii].accountSpent)
		  {  
			CELL(sp)=31; break;
		  }  
		}

		Watchdog=WatchdogN;

		while(len--)
		{ int rrr=0;
			do{     rrr=tread(fp,&BYTE(addr+res),1);
				if(rrr==0)
				if((fp&0x80000000)==0)
				if(eof(fp)) rrr=-2;
				if(fp&0x80000000)
				{   if(rrr==-1)
					{f_struct[t_ii].busy=0;
					f_struct[t_ii].fpos=-1;
					accSave(t_ii);
					return;
					}
				    if(rrr==0)
					{f_struct[t_ii].busy=0;
					f_struct[t_ii].fpos=-1;
					accSave(t_ii);
					return;
					}
/*
     {	char buffer[44];
	sprintf (buffer, "%c",BYTE(addr+res) );
	write(1,buffer,strlen(buffer));
     }
*/
				}
				if(fp==0) usleep(10000);
				}while(rrr==-1);
				  if(rrr!=1) break;



				if(BYTE(addr+res)=='\n') { res++; break; }
				if(BYTE(addr+res)==8)
				{ if(res>1) res--;
					res--;
			}	res++;
		}	ior=0;
		if(res)CELL(sp+4)=0xffffffff;
		else  CELL(sp+4)=0;
		while((BYTE(addr+res-1)<' ')&(res>0)) res--;

		CELL(sp+8)=res;

		if(!(fp&0x80000000))
		if(lstage>10)
		{
		    f_struct[t_ii].accountSpent+=res*FBiteCost;
		}

			CELL(sp)=ior;break;
  case sw_writefile: /*write-file*/

	sp+=8;FILEID(CELL(sp-8)) len=CELL(sp-4);
	addr=CELL(sp);CLIP
	twrite(fp,mem+addr,len);ior=0;
	CELL(sp)=ior;break;

  case sw_readfile: /*read-file*/

	sp+=4;FILEID(CELL(sp-4)) len=CELL(sp);
	addr=CELL(sp+4);CLIP
		if(lstage>10)
		if( len*FBiteCost > f_struct[t_ii].accountRecd-f_struct[t_ii].accountSpent)
		{
			CELL(sp)=31; break;
		}  

		Watchdog=WatchdogN;

	res=tread(fp,mem+addr,len);ior=0;
	CELL(sp+4)=res;
	if(!(fp&0x80000000))
	if(lstage>10)
	{
		f_struct[t_ii].accountSpent+=res*FBiteCost;
	}  
	CELL(sp)=ior;break;

  case sw_WatchdogADD:

	t=CELL(sp); sp+=4;
	t=MIN(t , f_struct[t_ii].accountRecd-f_struct[t_ii].accountSpent);
	Watchdog+=t;
	f_struct[t_ii].accountSpent+=t;
	break;

  case sw_typefilet: /*typefilet*/
	{  char fbuf[300];
		FILEID(CELL(sp))
		lseek(fp,0,SEEK_SET);
		res=read(fp,&fbuf,300);
		if(res<=300)
		twrite(fpoutput,fbuf,res);
 	}	break;

  case sw_help: /* help*/
     {  char fbuf[300]; UNS64 TimeC;

	TimeC=TimeCounter();
	printf("th= %lld %lld %lld\n", TimeC,helpTime,TimeC-helpTime );
//	if(helpIPAddr!=f_struct[t_ii].CIPAddr)
	if( (TimeC-helpTime)  > 0 )
	{
		accrual(f_struct[t_ii].fpos,(TimeC-helpTime)*2);
		helpTime=TimeCounter()+60000000;
		helpIPAddr=f_struct[t_ii].CIPAddr;
	}

	strcpy(fbuf,HomePath);
	strcat(fbuf,hiddir);
	strcat(fbuf,"/help.txt");

	t=open(fbuf,O_RDONLY|_O_BINARY,S_IRUSR | S_IWUSR );
	while( !eof(t) )
	{	res=read(t,&fbuf,300);
		if(res<=300)
		twrite(fpoutput,fbuf,res);
	}	close(t);
     }	break;

  case sw_deletefile: /*delete-file*/ sp+=4;len=CELL(sp-4);addr=CELL(sp);

	CLIP if(!make_name(filename,mem+addr,len)) {ior=-202;
	CELL(sp)=ior;break;} remove(filename); CELL(sp)=ior;break;

  case sw_repositionfile: /*reposition-file*/ sp+=8;FILEID(CELL(sp-8)) QFERR

	lseek(fp,CELL(sp),SEEK_SET);CELL(sp)=ior;break;

  case sw_fileposition: /*file-position*/ sp-=8;FILEID(CELL(sp+8)) QFERR

	CELL(sp+8)=lseek(fp, (off_t) 0, SEEK_CUR);CELL(sp+4)=0;
	CELL(sp)=ior;break;

  case sw_system: /*system */

	if(lstage>10)break;
	sp+=4;len=CELL(sp-4);addr=CELL(sp);
	CLIP if(!make_name(filename,mem+addr,len)) {ior=-202;
	CELL(sp)=ior;break;} ior=system(filename); CELL(sp)=ior;break;

  case sw_hiddir: /*system */

	if(lstage>10)break;
	strcpy( &BYTE( CELL(sp) + 1 + BYTE(CELL(sp)) ) , HomePath );
	strcat( &BYTE(CELL(sp)+1) , hiddir );
	BYTE( CELL(sp) ) = strlen( &BYTE( CELL(sp)+1 ) );
	sp+=4;	break;

  case sw_filesize: /* file-size*/ sp-=8;FILEID(CELL(sp+8))
     { int fpt;

	fpt=lseek(fp, (off_t) 0, SEEK_CUR);
	CELL(sp+8)=lseek(fp, (off_t) 0, SEEK_END);
	CELL(sp+4)=0;
	lseek(fp, (off_t) fpt, SEEK_SET);

      }break;

  case sw_restore: /* restore-system*/
      {  FILE *infile;

	sp+=4;len=CELL(sp-4);addr=CELL(sp);
	CLIP if(!make_name(filename,mem+addr,len)) {ior=-202;
	CELL(sp)=ior;break;}
	if((infile=fopen(filename,"rb")))  fread(&CELL(0),1,MEMSIZE,infile);

      }  zzz=1;

	printf("sw_restore\n");
	ip=FSTART; ireg=0;
	rp=MEMSIZE;
	break;

  case sw_telid:

	sp-=4;CELL(sp)=TELID; 	 break;

  case sw_ipadr:

	if(lstage>10)break;
	strncpy(inet_addr_buf,&BYTE(CELL(sp+4)),CELL(sp)&0xff);
	inet_addr_buf[CELL(sp)&0xff]=0;
	sp+=8;	break;

  case sw_connect:
	{ UNS32 tid=TELID;
	if(lstage>10)break;
	  for(TELID=6;TELID<FileFDMax;TELID++) { if(!fileids[TELID])break; }
	  if(TELID==FileFDMax){CELL(sp)=-1; TELID=tid; break;}
	  if(CCNN!=-1)closesocket(CCNN&(~0x80000000));
	  CCNN=c_lconnect(CELL(sp))|0x80000000;
	  fileids[TELID]=CCNN;
	  CELL(sp)=TELID;
	}	break;

  case sw_tellen:
	{ int nchar;

	  sp-=4;
	  if( ioctlsocket( CCNN&(~0x80000000), FIONREAD, &nchar ) == -1)
		{ CELL(sp)=0; break;
		} CELL(sp)=nchar;

	} break;

  case sw_telp:
     {
	int telpMAX = CELL(sp);
	int telpBUF = CELL(sp+4);
	for(;;)
	{ char cc; int rrr; int nchar;

		if( ioctlsocket( CCNN&(~0x80000000), FIONREAD, &nchar ) == -1)
		{  CELL(sp)=-1;
			break; }

		if(nchar==0)  { CELL(sp)=0;
//			  twrite(fileids[CELL(OUTPUT)],"#",1); //
		  break; }
		rrr=tread(CCNN,&cc,1);
		if(rrr==0) { CELL(sp)=telpCount;
		twrite(fpoutput,"%",1); //
		break;}
		if(rrr==-1) { CELL(sp)=telpCount;
		twrite(fpoutput,"&",1); //
		break;
		}
		if(calltelp)
		{
			if( cc=='\n' || telpCount==telpMAX )
			{ CELL(sp)=telpCount;
			  telpCount=0;
			  calltelp=0;
//			  twrite(fpoutput,"}",1); //
			  break;
			}
			BYTE(telpBUF+telpCount)=cc;
//			twrite(fpoutput,&cc,1); //
			telpCount++;
		}
		else
		{	if(tplast=='\n' && cc=='~')
			{	calltelp=1;
//				twrite(fpoutput,"{",1); //
			}
			else
			{
				twrite(fpoutput,&cc,1);
				tplast=cc;
			}
		}
	}
     }
	    	  break;

  case sw_setpabdir: /* pabdir!*/

	if(lstage>10)break;
	pabdir= &BYTE(CELL(sp));
	sp+=4;break;

  case sw_ms: /* ms*/

	t=CELL(sp)*10;
	while(t--) usleep(100);
	sp+=4;break;

  case sw_TimeCounter: /* timecounter*/

	dtmp=TimeCounter();
	sp-=8;
	CELL(sp+4)=dtmp;
	CELL(sp)=dtmp>>32;
	break;

  case sw_modexp: /* modexp*/

	{ UNS64 xx, rez, kmod,kexp ;


	kmod=CELL(sp);	kmod<<=32;
	kmod|=CELL(sp+4);

	sp+=8;

	kexp=CELL(sp);	kexp<<=32;
	kexp|=CELL(sp+4);

	sp+=8;

	xx=CELL(sp);	xx<<=32;
	xx|=CELL(sp+4);

	rez=modexp(xx,kexp,kmod);

	CELL(sp+4)=rez;
	CELL(sp)=rez>>32;

	break;
	}

  case sw_gcd: /* gcd*/
    {  UNS64 rez, ke,kp ,kq,km ;

	kq=CELL(sp);
	kp=CELL(sp+4);
	ke=CELL(sp+8);

	km=(kp-1)*(kq-1);

	rez=GCD(ke,(kp-1)*(kq-1));

	CELL(sp+8)=km;
	CELL(sp+4)=km>>32;
	CELL(sp)=rez;

	break;
    }

  case sw_modinv: /* modinv*/
	{  UNS64 rez, ka ,km;


	km=CELL(sp);	km<<=32;
	km|=CELL(sp+4);

	sp+=8;

	ka=CELL(sp);	ka<<=32;
	ka|=CELL(sp+4);


	rez=modinv(ka,km);
	CELL(sp+4)=rez;
	CELL(sp)=rez>>32;
	break;
	}

  case sw_ToSimple:	CELL(sp)=ToSimple(CELL(sp));	break;

  case sw_fnumber: /* fnumber? */

	{ char * pEnd; float ff;

	 len=CELL(sp);addr=CELL(sp+4);
	 make_name(filename,mem+addr,len);
	 if( (filename[len-1]|0x20)=='e') { filename[len]='0'; filename[len+1]='\0';  }
	 ff = strtof (filename,&pEnd);
	 while (isspace (*pEnd)) pEnd++;
	 if(*pEnd == '\0')
	 {	FCELL(sp+4)=ff; CELL(sp)=-1;  }
	 else
	 {	sp-=4; CELL(sp)=0; }
	}	break;

  case sw_fadd:	/* f+ */	FCELL(sp+4)+=FCELL(sp);sp+=4;break;
  case sw_fsub:	/* f- */	FCELL(sp+4)-=FCELL(sp);sp+=4;break;
  case sw_fmul:	/* f* */	FCELL(sp+4)*=FCELL(sp);sp+=4;break;
  case sw_fdiv:	/* f/ */

	if(FCELL(sp)==0.0)	{ FCELL(sp+4)=999999.; sp+=4;break; }
	FCELL(sp+4)/=FCELL(sp);	sp+=4;break;

  case sw_f2ud:	/* f>ud */

	dtmp= FCELL(sp);
	CELL(sp)=dtmp;  sp-=4;
	CELL(sp)=dtmp>>32;break;

  case sw_u2f:	/* u>f */	FCELL(sp)=CELL(sp);	break;

  case sw_fprint: /* f. */
     {	char buffer[44];
	sprintf (buffer,"%4.8f ", FCELL(sp)); sp+=4;
	twrite(fpoutput,buffer,strlen(buffer));
	break;
     }

  case sw_fcells: /* fcells */	CELL(sp)*=sizeof(float); break;
  case sw_Stage:	sp-=4; CELL(sp)=lstage; break;
  case sw_StageAdd:

	if(lstage>10)	break;
	if(stage==0)	memcpy(mainmem,pmainmem,MEMSIZE);
	stage++;	break;

  case sw_getmyip:	sp-=4; CELL(sp)=getmyip();	break;

  case sw_hostname2ip:

	len=CELL(sp);addr=CELL(sp+4); sp+=4;
	make_name(filename,mem+addr,len);
	CELL(sp)=hostname2ip(filename);
	break;

  case sw_getacc:

	dtmp=f_struct[t_ii].accountRecd-f_struct[t_ii].accountSpent;
	sp-=4; CELL(sp)=dtmp;
	sp-=4; CELL(sp)=dtmp>>32;break;

  case sw_getbarter:

	dtmp=f_struct[t_ii].data.barter-f_struct[t_ii].barterSpent;
	sp-=4; CELL(sp)=dtmp;
	sp-=4; CELL(sp)=dtmp>>32;break;

  case sw_FBiteCost:

	dtmp= (((UNS64)CELL(sp)<<32) + (UNS64)CELL(sp+4)) * FBiteCost;
	CELL(sp+4)=dtmp;
	CELL(sp)=dtmp>>32;break;

  case sw_render: // ( d n -- )
     {  UNS64 render;
	dtmp=f_struct[t_ii].accountRecd-f_struct[t_ii].accountSpent;
	
	render=CELL(sp+4);	render<<=32;
	render|=CELL(sp+8);

	render=MIN(dtmp,render);

	accrualn(CELL(sp),render);

	f_struct[t_ii].accountSpent+=render;

	sp+=4*3;

     }	break;

  case sw_frombarter:
     {	UNS64 barter, getbarter;

	barter=f_struct[t_ii].data.barter;
	barter-=f_struct[t_ii].barterSpent;

	getbarter=CELL(sp);
	getbarter<<=32;
	getbarter|=CELL(sp+4);

	getbarter=MIN(barter,getbarter);
	accrual(f_struct[t_ii].fpos,getbarter);

	f_struct[t_ii].barterSpent+=getbarter;

	CELL(sp+4)=getbarter;
	CELL(sp)=getbarter>>32; sp-=4; 
	CELL(sp)=f_struct[t_ii].data.barterN;

	break;
     }

  case sw_GetAccKey: // (  -- AccKey )

        sp-=4;
	AccKey+=TimeCounter();
//	AccKey=7777;
	CELL(sp)=AccKey;
	break;

  case sw_kaccount: // ( nlog kpass -- )
     {  UNS32 passw; UNS32 fpos;
	passw=CELL(sp+4);
	fpos=CELL(sp)*sizeof(f_data_t);
	sp+=4;
	CELL(sp)=-1;
	if(posis(fpos)) break;
	f_struct[t_ii].fpos=fpos;
	accSave(t_ii);
	pthread_mutex_lock(&mutexacc);
	lseek(accdat, fpos, SEEK_SET);
	read(accdat, (char *) &f_struct[t_ii].data , sizeof(f_data_t));
	if(  modexp(f_struct[t_ii].data.pass,AccKey,0x100000000)==passw)
	{
		f_struct[t_ii].accountRecd=f_struct[t_ii].data.account;
		CELL(sp)=0;
	}else
	{	f_struct[t_ii].data.account=0;
		f_struct[t_ii].data.barter=0;
		f_struct[t_ii].fpos=-1;
	}
		pthread_mutex_unlock(&mutexacc);
     }	break;

  case sw_AccountNew:

	f_struct[t_ii].data.pass=CELL(sp);
	f_struct[t_ii].data.state=1;
	f_struct[t_ii].data.pas2=0;
	accSave(t_ii);
	pthread_mutex_lock(&mutexacc);

	f_struct[t_ii].fpos=lseek(accdat, (off_t) 0, SEEK_END);
	if((f_struct[t_ii].fpos&~1)==0) f_struct[t_ii].data.account=1000000000000;
	write(accdat, (char *) &f_struct[t_ii].data , sizeof(f_data_t));
	f_struct[t_ii].accountRecd=f_struct[t_ii].data.account;

	pthread_mutex_unlock(&mutexacc);

	f_struct[t_ii].pp=1;

  case sw_cnumber:	sp-=4;	CELL(sp)=f_struct[t_ii].fpos/sizeof(f_data_t);	break;

  case sw_passadd:

	f_struct[t_ii].data.pass+=CELL(sp);	sp+=4;

	pthread_mutex_lock(&mutexacc);

	lseek(accdat, (off_t) f_struct[t_ii].fpos, SEEK_SET);

	write(accdat, (char *) &f_struct[t_ii].data , sizeof(f_data_t));

	pthread_mutex_unlock(&mutexacc);

	break;

  case sw_hostpassadd:

	f_hosts.pass+=CELL(sp);	sp+=4;
	lseek(HostsDat,CountHostPos,SEEK_SET);
	write(HostsDat, (char *) &f_hosts , sizeof(f_hosts_t));
	break;

  case sw_OldAcc:
		if(lstage>10)break;
	if(LoginSearch(hostname2ip(inet_addr_buf),CELL(sp)))
	{ sp-=8;
	  CELL(sp+8)=f_hosts.number;
	  CELL(sp+4)=f_hosts.pass;
	  CELL(sp)=-1; break;
	} CELL(sp)=0; break;

  case sw_NOldAcc:
		if(lstage>10)break;
	  t=CELL(sp)*sizeof(f_hosts_t);
	  if(t<lseek(HostsDat, (off_t) 0, SEEK_END))
	  {	    char buffer[44];
		  lseek(HostsDat, t, SEEK_SET);
		  read(HostsDat, (char *) &f_hosts , sizeof(f_hosts_t));
		  	twrite(fpoutput,f_hosts.ipaddr,strlen(f_hosts.ipaddr));
			sprintf (buffer, " %d %d", f_hosts.port,f_hosts.number);
			twrite(fpoutput,buffer,strlen(buffer));
	  CELL(sp)=0; break;
	} CELL(sp)=-1; break;

  case sw_NumConnect:
		if(lstage>10)break;
	  t=CELL(sp)*sizeof(f_hosts_t);
	  if(t<lseek(HostsDat, (off_t) 0, SEEK_END))
	  {
		  lseek(HostsDat, t, SEEK_SET);
		  read(HostsDat, (char *) &f_hosts , sizeof(f_hosts_t));
		  strncpy(inet_addr_buf,f_hosts.ipaddr,strlen(f_hosts.ipaddr));
		  CELL(sp)=f_hosts.port;
		   break;
	  }
	  CELL(sp)=0;
		   break;
	break;
  case sw_GETOLDACC:
	  sp-=8;
	  	  CELL(sp+4)=f_hosts.number;
	  	  CELL(sp)=f_hosts.pass;
	   break;
  case sw_NPort:	sp-=4; CELL(sp)=NPort;	break;

  case sw_barterAdd:
    {	UNS32 ii=0; UNS32 fpos;  UNS64 barter; UNS32 barterN;
	if(lstage>10)break;
	barterN=CELL(sp);
	fpos=CELL(sp+4)*sizeof(f_data_t);
	barter=CELL(sp+8);
	barter<<=32;
	barter|=CELL(sp+12);
	sp+=16;

	pthread_mutex_lock(&mutexacc);

	if(fpos>=lseek(accdat, (off_t) 0, SEEK_END))goto barterAdd_end;

	lseek(accdat, (off_t) fpos, SEEK_SET);

	for(;ii<f_structMax;ii++)
	{
	  if(f_struct[ii].fpos==fpos)
	  {
		f_struct[ii].data.barterN=barterN;
		f_struct[ii].data.barter+=barter-f_struct[ii].barterSpent;
		f_struct[ii].barterSpent=0;
		write(accdat, (char *) &f_struct[ii].data , sizeof(f_data_t));
		goto barterAdd_end;
	  }
	}
	{	f_data_t data;
		read(accdat, (char *) &data , sizeof(f_data_t));
		data.barterN=barterN;
		data.barter+=barter;
		lseek(accdat, (off_t) fpos, SEEK_SET);
		write(accdat, (char *) &data , sizeof(f_data_t));
	}

barterAdd_end:  pthread_mutex_unlock(&mutexacc);
	printf("barter=%llu\n", f_struct[ii].data.barter);
	break;
    }

  case sw_s2cln:

	sp+=4;
	if(LoginSearch(CELL(sp),CELL(sp-4)))
	{ CELL(sp)=f_hosts.number;   break;
	} CELL(sp)=-1; break;
	break;

  case sw_accrual:
	if(lstage>10)break;
	{ UNS64 account;
		account=CELL(sp+4);
		account<<=32;
		account|=CELL(sp+8);
		accrualn(CELL(sp),account);
		sp+=12;
	} break;

  case sw_hostadd:
     {	UNS32 pw,port,number;
	if(lstage>10)break;

	pw=CELL(sp+8); 	number=CELL(sp+4);
	port=CELL(sp);

	sp+=12;

	memset(f_hosts.ipaddr, 0, sizeof(f_hosts.ipaddr));
	strcpy(f_hosts.ipaddr ,inet_addr_buf);
	f_hosts.pass=pw;
	f_hosts.port=port;
	f_hosts.number=number;

	CountHostPos=lseek(HostsDat, (off_t) 0, SEEK_END);
	write(HostsDat, (char *) &f_hosts , sizeof(f_hosts_t));

     }	break;

  case sw_bid:
		if((int)f_struct[t_ii].fpos<0) {CELL(sp)=-1;break;  }
		 CELL(sp)=BidExec(&BYTE(CELL(sp)),f_struct[t_ii].fpos/sizeof(f_data_t) );
 	 break;

  case sw_ztstp:
	 CELL(sp)=ToSimple(CELL(sp));
 	 break;

  case sw_hi:
	twrite(fpoutput,"Hello!!!\n",9);
	break;
  defalt: break;
   }
	if(ireg&0xC000)/*return*/ {
		ip=CELL(rp);
		rp+=4;
	} 

  }
 }
}

make_image()
{ 	MEMDEF
	UNS32 shere;
 HERE=0;
 HERE=vocs_beg;
 CELL(HANDLER)=0;
 CELL(ATIB)=TTIB;
 CELL(CONTEXT)=VFORTH;
 CELL(CONTEXT+4)=0;
 CELL(CURRENT)=VFORTH;
 CELL(INPUT)=0;
 CELL(OUTPUT)=1;
 current=0;

write(1,"make_image\n",11);
PRIM("NOOP",sw_nop);
PRIM("CHARS",sw_nop);
PRIM("SWAP",sw_swap);
PRIM("ROT",sw_rot);
PRIM("0=",sw_0equ);
PRIM("0<>",sw_0Nequ);
PRIM("0<",sw_0LT);
PRIM("0>",sw_0GT);
PRIM("=",sw_equ);
PRIM("<>",sw_nequ);
PRIM("NEGATE",sw_negate);
PRIM("INVERT",sw_invert);
PRIM("UM*",sw_ummul);
PRIM("C@",sw_Cload);
PRIM("W@",sw_Wload);
PRIM("@",sw_load);
PRIM("ON",sw_on);
PRIM("0!",sw_0ST);
PRIM("1+",sw_1add);
PRIM("2+",sw_2add);
PRIM("1-",sw_1sub);
PRIM("2-",sw_2sub);
PRIM("2*",sw_2mul);
PRIM("2/",sw_2div);
PRIM("+",sw_add);
PRIM("-",sw_sub);
PRIM("*",sw_mul);
PRIM("/",sw_div);
PRIM("MOD",sw_mod);
PRIM("CELL/",sw_celldiv);
PRIM("CELLS",sw_cells);
PRIM("AND",sw_and);
PRIM("OR",sw_or);
PRIM("XOR",sw_xor);
PRIM("U<",sw_uLT);
PRIM("<",sw_LT);
PRIM("U>",sw_uGT);
PRIM(">",sw_GT);
PRIM("UMIN",sw_umin);
PRIM("UMAX",sw_umax);
PRIM("MIN",sw_min);
PRIM("MAX",sw_max);
PRIM("LSHIFT",sw_lshift);
PRIM("RSHIFT",sw_rshift);
PRIM("UM/MOD",sw_umSmod);
PRIM("+CY",sw_addcy);
PRIM("D+",sw_Dadd);
PRIM("D-",sw_Dsub);
PRIM("UD*",sw_UDmul);
PRIM("UD/",sw_UDdiv);
PRIM("UD<",sw_UDLT);
PRIM("DROP",sw_drop);
PRIM("2DROP",sw_2drop);
PRIM("LIT,",sw_litcomp);
PRIM(",",sw_comp);
PRIM("W,",sw_Wcomp);
PRIM("C,",sw_Ccomp);
PRIM("ALLOT",sw_allot);
PRIM("EXECUTE",sw_execute);
PRIM("COMPILE,",sw_compile);
PRIM("IMMEDIATE",sw_immediate);
PRIM("NAME>",sw_FromName);
PRIM(">BODY",sw_2add);
PRIM("SP@",sw_spload);
PRIM("SP!",sw_spST);
PRIM("RP@",sw_rpload);
PRIM("RP!",sw_rpST);

PRIM("THROW",sw_throw);
PRIM("W,",sw_Wcomp);
PRIM("!",sw_ST);
PRIM("+!",sw_addST);
PRIM("C!",sw_cST);
PRIM("W!",sw_wST);
PRIM("2!",sw_2ST);
PRIM("2@",sw_2load);
PRIM("2SWAP",sw_2swap);
PRIM("C!A",sw_cSTa);
PRIM("?DUP",sw_qdup);
PRIM("DUP",sw_dup);
PRIM("2DUP",sw_2dup);
PRIM("OVER",sw_over);
PRIM("R@",sw_rload);
PRIM("R>",sw_rGT);
PRIM(">R",sw_GTr);
PRIM("KEY",sw_key);
PRIM("EMIT",sw_emit);
PRIM("KEY?",sw_keyq);
PRIM("TYPE",sw_type);
PRIM("HI",sw_hi);
PRIM("BYE",sw_bye);
PRIM("H.",sw_hdot);
PRIM("DEC.",sw_decdot);
PRIM("#CHDIR",sw_chdir);
PRIM("$DIR",sw_dir);
PRIM("COUNT",sw_count);
PRIM("COMPARE",sw_Compare);
PRIM("PARSE-NAME",sw_NextWord);
PRIM("PARSE",sw_Parse);
PRIM("S\"", sw_SQ); IMMED();
PRIM(".\"", sw_PQ); IMMED();
PRIM("SOURCE",sw_Source);
PRIM("SOURCE!",sw_SourceST);
PRIM("SEARCH-WORDLIST",sw_SearchWordList);
PRIM("SFIND",sw_SFind);
PRIM("SHEADER",sw_SHeader);

PRIM("BEGIN",sw_BEGIN); IMMED();
PRIM("AGAIN",sw_AGAIN); IMMED();
PRIM("UNTIL",sw_UNTIL); IMMED();
PRIM("AHEAD",sw_AHEAD); IMMED();
PRIM("IF",sw_IF); IMMED();
PRIM("WHILE",sw_WHILE); IMMED();
PRIM("THEN",sw_THEN); IMMED();
PRIM("ELSE",sw_ELSE); IMMED();
PRIM("REPEAT",sw_REPEAT); IMMED();

PRIM("OPEN-FILE",sw_openfile);
PRIM("CLOSE-FILE",sw_closefile);

PRIM("WRITE-LINE",sw_writeline);
PRIM("READ-LINE",sw_readline);

PRIM("WRITE-FILE",sw_writefile);
PRIM("READ-FILE",sw_readfile);
PRIM("WDOG+",sw_WatchdogADD);
PRIM("TYPEFILET",sw_typefilet);
PRIM("HELP",sw_help);
PRIM("DELETE-FILE",sw_deletefile);
PRIM("REPOSITION-FILE",sw_repositionfile);
PRIM("FILE-POSITION",sw_fileposition);
PRIM("SYSTEM",sw_system);
PRIM("+HIDDIR",sw_hiddir);
PRIM("FILE-SIZE",sw_filesize);
PRIM("RESTORE-SYSTEM",sw_restore);
//PRIM("RS",sw_restore);
PRIM("TELID",sw_telid);
PRIM("IPADR!",sw_ipadr);
PRIM("CONNECT",sw_connect);
PRIM("TELN",sw_tellen);
PRIM("TEL.",sw_telp);
PRIM("PABDIR!",sw_setpabdir);
PRIM("MS",sw_ms);
PRIM("TIMECR",sw_TimeCounter);
PRIM("MODEXP",sw_modexp);
PRIM("GCD",sw_gcd);
PRIM("MODINV",sw_modinv);
PRIM(">SIMPLE",sw_ToSimple);
PRIM("FNUMBER?",sw_fnumber);
PRIM("F+",sw_fadd);
PRIM("F-",sw_fsub);
PRIM("F*",sw_fmul);
PRIM("F/",sw_fdiv);
PRIM("F>UD",sw_f2ud);
PRIM("U>F",sw_u2f);
PRIM("F.",sw_fprint);
PRIM("FCELLS",sw_fcells);
PRIM("STAGE",sw_Stage);
PRIM("STAGE+",sw_StageAdd);
PRIM("GETMYIP",sw_getmyip);
PRIM("H2IP",sw_hostname2ip);
PRIM("GETACC",sw_getacc);
PRIM("GETBARTER",sw_getbarter);
PRIM("FBCOST*",sw_FBiteCost);
PRIM("DRENDR",sw_render);
PRIM("FROMBARTER",sw_frombarter);
PRIM("GETACCK",sw_GetAccKey);
PRIM("KACCOUNT",sw_kaccount);
PRIM("ACCNEW",sw_AccountNew);
PRIM("CNUMBER",sw_cnumber);
PRIM("PASSADD",sw_passadd);
PRIM("HPASSADD",sw_hostpassadd);
PRIM("OLDACC?",sw_OldAcc);
PRIM("NOLDACC",sw_NOldAcc);
PRIM("NUMCONNECT",sw_NumConnect);
PRIM("GETOLDACC",sw_GETOLDACC);
PRIM("NPORT",sw_NPort);
PRIM("BARTERADD",sw_barterAdd);
PRIM("S2CLN",sw_s2cln);
PRIM("ACC++",sw_accrual);
PRIM("HOST++",sw_hostadd);
PRIM("ZBID",sw_bid);
PRIM("ZTST",sw_ztstp);
CONSTANT("CELL",4);
CONSTANT(">IN",TO_IN);
CONSTANT("STATE",STATE);
CONSTANT("[EXIT]",0x8001);
CONSTANT("C/L",TIB_SAZE);
CONSTANT("ATIB",ATIB);
CONSTANT("TTIB",TTIB);
CONSTANT("#TIB",NTIB);
CONSTANT("DP",DPOINT);
CONSTANT("CURRENT",CURRENT);
CONSTANT("CONTEXT",CONTEXT);
CONSTANT("LAST",LAST);
CONSTANT("FORTH-WORDLIST",VFORTH);
CONSTANT("CREATE-CODE",sw_ip<<1|0x4001);
CONSTANT("CONSTANT-CODE",sw_lit<<1|0x4001);

CONSTANT("R/O",O_RDONLY);
CONSTANT("W/O",O_WRONLY);
CONSTANT("R/W",O_RDWR);
CONSTANT("O_CREAT",O_CREAT);

HERE+=0x300;

CONSTANT("PAD",HERE-0x100);

VARIABLE("DPL");
VARIABLE("BASE");
VARIABLE("SP0");
CONSTANT("STDOUT",OUTPUT);

HEADER("DECIMAL"); LIT(10); CC("BASE");  CC("!"); F_END();
HEADER("HEX");     LIT(16); CC("BASE");  CC("!"); F_END();
HEADER("["); CC("STATE");  CC("0!"); F_END();  IMMED();
HEADER("]"); CC("STATE");  CC("ON"); F_END();
HEADER("TIB"); CC("ATIB");  CC("@"); F_END();
VARIABLE("WINTERMV");
HEADER("WINTERM"); LIT(-1); CC("WINTERMV");  CC("!"); F_END();
HEADER("LINTERM"); LIT(0);  CC("WINTERMV");  CC("!"); F_END();

HEADER("CATCH");

WCOM(sw_to_catch<<1|1);
WCOM(sw_from_catch<<1|1);
F_END();

HEADER("TST"); LIT(0x33); CC("H.");
 F_END();

 HEADER("DNEGATE"); // ( d1 --- d2)
 // Negate the top double number on the stack.
 CC(">R"); CC("NEGATE"); CC("R>"); CC("NEGATE"); CC("OVER"); CC("0<>");  CC("+");
 F_END();


 HEADER("DIGIT?"); // ( c -- 0| c--- n -1)
 // Convert character c to its digit value n and return true if c is a
 // digit in the current base. Otherwise return false.
 LIT('0'); CC("-"); CC("DUP"); CC("0<"); IF(); CC("DROP"); LIT(0); WCOM(0x8001); THEN();
 CC("DUP"); LIT(9); CC(">"); CC("OVER"); LIT(17); CC("<"); CC("AND"); IF(); CC("DROP"); LIT(0); WCOM(0x8001); THEN();
 CC("DUP"); LIT(9); CC(">");
  IF(); LIT('0'); CC("+"); LIT(0x20); CC("OR");
	LIT('0'); CC("-"); LIT(0x27); CC("-");
  THEN();
 CC("DUP"); CC("BASE"); CC("@"); CC("<"); CC("0="); IF(); CC("DROP"); LIT(0); WCOM(0x8001); THEN();
 LIT(-1);
 F_END();

 HEADER(">NUMBER"); // ( ud1 c-addr1 u1 --- ud2 c-addr2 u2 )
 // Convert the string at c-addr with length u1 to binary, multiplying ud1
 // by the number in BASE and adding the digit value to it for each digit.
 // c-addr2 u2 is the remainder of the string starting at the first character
 // that is no digit.
   BEGIN();
   CC("DUP");
   WHILE();
    CC("1-"); CC(">R");
    CC("COUNT"); CC("DIGIT?"); CC("0=");
    IF();
    CC("R>"); CC("1+"); CC("SWAP"); CC("1-"); CC("SWAP");  WCOM(0x8001);
    THEN();
    CC("SWAP"); CC(">R");
    CC(">R");
    CC("SWAP"); CC("BASE"); CC("@"); CC("UM*"); CC("ROT"); CC("BASE"); CC("@");
    CC("*"); LIT(0); CC("SWAP"); CC("D+"); // Multiply ud by base.
    CC("R>"); LIT(0); CC("D+");           // Add new digit.
    CC("R>"); CC("R>");
   REPEAT();
   F_END();

 HEADER("CONVERT"); // ( ud1 c-addr1 --- ud2 c-addr2)
 // Convert the string starting at c-addr1 + 1 to binary. c-addr2 is the
 // address of the first non-digit. Digits are added into ud1 as in >NUMBER
 CC("1-"); LIT(-1); CC(">NUMBER"); CC("DROP");
 F_END();

 HEADER("NUMBER?"); // ( c-addr n ---- d f)
 // Convert the counted string at c-addr to a double binary number.
 // f is true if and only if the conversion was successful. DPL contains
 // -1 if there was no point in the number, else the position of the point
 // from the right. Special prefixes: # means decimal, $ means hex.
 LIT(-1); CC("DPL"); CC("!");
 CC("BASE"); CC("@"); CC(">R");
// CC("COUNT");
 CC("OVER"); CC("C@"); LIT(45); CC("="); CC("DUP"); CC(">R"); IF(); CC("1-"); CC("SWAP"); CC("1+"); CC("SWAP"); THEN(); // Get any - sign
 CC("OVER"); CC("C@"); LIT(36); CC("="); IF(); LIT(16); CC("BASE"); CC("!"); CC("1-"); CC("SWAP"); CC("1+"); CC("SWAP"); THEN();   // $ sign for hex.
 CC("OVER"); CC("C@"); LIT(35); CC("="); IF(); LIT(10); CC("BASE"); CC("!"); CC("1-"); CC("SWAP"); CC("1+"); CC("SWAP"); THEN();   // # sign for decimal
 CC("DUP");  CC("0>"); CC("0=");
 IF();  CC("R>"); CC("DROP"); CC("R>"); CC("BASE"); CC("!"); LIT(0); WCOM(0x8001);
 THEN();   // Length 0 or less?
 CC(">R"); CC(">R"); LIT(0); LIT(0); CC("R>"); CC("R>");
   BEGIN();
 CC(">NUMBER");
 CC("DUP"); IF(); CC("OVER"); CC("C@"); LIT(46); CC("="); IF(); CC("1-"); CC("DUP");
 CC("DPL"); CC("!"); CC("SWAP"); CC("1+"); CC("SWAP"); ELSE(); // handle point.
 CC("R>"); CC("DROP"); CC("R>"); CC("BASE"); CC("!"); LIT(0); WCOM(0x8001); THEN();  // Error if anything but point
        THEN();
   CC("DUP"); CC("0="); UNTIL(); CC("2DROP"); CC("R>"); IF(); CC("DNEGATE"); THEN();
   CC("R>"); CC("BASE"); CC("!"); LIT(-1);
   F_END();

HEADER("DEPTH");
  CC("SP0"); CC("@"); CC("SP@");  CC("-");
  CC("CELL/"); CC("1-");
  F_END();

HEADER("?STACK");
CC("DEPTH");  CC("0<");
IF(); LIT(-4); CC("THROW");
THEN();
F_END();

 HEADER("INTERPRET");
// TP("INTERPRET\n");
 BEGIN();
 CC("PARSE-NAME"); CC("DUP");
 WHILE();
 CC("SFIND"); CC("?DUP");
    IF();
    CC("STATE"); CC("@"); CC("=");
	IF();  CC("COMPILE,");
	ELSE();
//	TP("st=");  CC("DUP");  CC("H.");
	CC("EXECUTE");
	THEN();
    ELSE();

	CC("DUP"); CC(">R");
	CC("NUMBER?");
	IF();  CC("R>"); CC("DROP");

	CC("DPL"); CC("@"); CC("1+");
	IF(); 	CC("STATE"); CC("@");
		IF(); CC("SWAP"); CC("LIT,"); CC("LIT,");
		THEN();
	ELSE(); CC("DROP"); CC("STATE"); CC("@"); IF(); CC("LIT,"); THEN();
	THEN();
	ELSE();  CC("2SWAP"); CC("2DROP");  CC("R@"); CC("-");  CC("+");  CC("R>");
		CC("FNUMBER?"); CC("0=");  IF();  LIT(-13); CC("THROW"); THEN();
		CC("STATE"); CC("@");  IF(); CC("LIT,"); THEN();
	THEN();
    THEN();

 CC("?STACK");
 REPEAT();
 CC("2DROP");
 F_END();

 HEADER("EVAL");
 CC("SOURCE!");
 CC(">IN");   CC("0!");
 CC("INTERPRET");
 F_END();

 HEADER(":");
 CC("STATE"); CC("ON");
 CC("PARSE-NAME");
// TP("PARSE-NAME=");  CC("2DUP"); CC("TYPE");
 CC("SHEADER");
 F_END();

 HEADER("EXIT");  IMMED();
 CC("[EXIT]"); CC("W,");
 F_END();

 HEADER(";");  IMMED();
 WCOM(sw_femd<<1|1);
 CC("STATE"); CC("0!");
 F_END();

 HEADER("'");
 CC("PARSE-NAME"); CC("SFIND");  CC("0=");
 IF(); LIT(-321); CC("THROW");
 THEN();
 F_END();

 HEADER("[']"); IMMED();
 CC("'"); CC("LIT,");
 F_END();

 HEADER("(");   IMMED();
  LIT(')'); CC("PARSE"); CC("2DROP");
 F_END();

 HEADER(".(");   IMMED();
  LIT(')'); CC("PARSE"); CC("TYPE");
 F_END();

 HEADER("\\");   IMMED();
  LIT(0); CC("PARSE"); CC("2DROP");
 F_END();

 HEADER("CR");
  LIT('\n');  CC("EMIT");
  CC("WINTERMV"); CC("@");
  IF(); LIT('\r');  CC("EMIT");
  THEN();
 F_END();

 HEADER("SPACE");
  LIT(' ');  CC("EMIT");
 F_END();

 HEADER("MU/MOD");
 // ( ud u --- urem udquot )  Divide unsigned double number
 //ud by u and return a double quotient and G a single remainder.
 CC(">R"); LIT(0); CC("R@"); CC("UM/MOD"); CC("R>");
       CC("SWAP"); CC(">R"); CC("UM/MOD"); CC("R>");
F_END();

 HEADER("DABS"); // ( d --- ud)  ud is the absolute value of d.
	CC("DUP"); CC("0<"); IF(); CC("DNEGATE"); THEN();
F_END();

 VARIABLE("HLD");

 HEADER("HOLD"); // ( c ---)  Insert character c into the  numerical  conver- sion buffer.
 LIT(-1);  CC("HLD"); CC("+!"); CC("HLD"); CC("@"); CC("C!");
 F_END();

 HEADER("#");  // ( ud1 --- ud2)  Extract the rightmost digit of ud1 and put
 // it into the numerical G conversion buffer.
 CC("BASE"); CC("@"); CC("MU/MOD"); CC("ROT"); CC("DUP");
  LIT(9); CC(">");
  IF(); LIT(7); CC("+");
  THEN(); LIT(48); CC("+"); CC("HOLD");
 F_END();

 HEADER("#S");  // ( ud --- 0 0 )  Convert ud by repeated use of # until ud is zero.
   BEGIN(); CC("#"); CC("2DUP"); CC("OR"); CC("0="); UNTIL();
 F_END();

 HEADER("SIGN"); // (  n  ---)  Insert a - sign in the numerical conversion buffer if n is negative.
	CC("0<"); IF(); LIT(45); CC("HOLD"); THEN();
 F_END();

 HEADER("<#"); // ( --- )  Reset the numerical conversion buffer.
 CC("PAD"); CC("HLD"); CC("!");
 F_END();

 HEADER("#>"); // ( ud --- addr u )  Discard ud  and  give  the  address  and  length of the numerical conversion G buffer.
 CC("2DROP");  CC("HLD"); CC("@"); CC("PAD"); CC("OVER"); CC("-");
 F_END();

 HEADER("(D.)");
	CC("SWAP"); CC("OVER"); CC("DABS"); CC("<#"); CC("#S"); CC("ROT"); CC("SIGN");
	CC("#>");
 F_END();

HEADER("D."); // ( d --- )  Type the double number d to the terminal.
	CC("(D.)"); CC("TYPE"); CC("SPACE");
 F_END();

HEADER("U."); // ( u ---)  Type the unsigned number u to the terminal.
	LIT(0); CC("D.");F_END();

 HEADER("S>D");
	CC("DUP");  CC("0<"); F_END();

 HEADER("."); // ( n ---)  Type the signed number n to the terminal.
	CC("S>D"); CC("D."); F_END();

 VARIABLE("SID");


 HEADER("SOURCE-ID"); CC("SID");  CC("@");
 F_END();

HEADER("\\EOF");
	CC("SOURCE-ID"); CC("FILE-SIZE"); CC("DROP");
	CC("SOURCE-ID"); CC("REPOSITION-FILE"); CC("DROP");
	CC("\\");
F_END();


HEADER("ACCEPT"); // TP("READ-LINE");
	LIT(0); CC("READ-LINE"); CC("2DROP");
F_END();

HEADER("REFILL");
CC("TIB"); CC("C/L"); CC("SOURCE-ID");
// CC("DUP"); CC("H.");   TP("READ-LINE\n");
   CC("READ-LINE");  CC("DROP");
   IF();  CC("TIB"); CC("SWAP"); CC("SOURCE!");  LIT(-1);
   ELSE(); CC("DROP"); LIT(0);
   THEN();
   CC(">IN"); CC("0!");
   F_END();

   HEADER("FQUIT");
//   TP("BEGIN QUIT\n");
   BEGIN();  CC("REFILL");
   WHILE();  CC("INTERPRET"); // TP(" OK"); CC("CR");
   REPEAT();
   F_END();
   HEADER("QUIT");
//   TP("BEGIN QUIT\n");
   BEGIN();  CC("REFILL");
   WHILE();  CC("INTERPRET");
   CC("STATE"); CC("@"); CC("0="); IF();  TP(" OK"); CC("CR"); THEN();
   REPEAT();
   F_END();
  shere=HERE;
  HERE+=1000;

 FSTART=HERE;
 TP("Forth\n");
 CC("STATE"); CC("0!");
 CC("SID"); CC("0!");
 CC("SP@"); CC("SP0"); CC("!");
 CC("DECIMAL");
 //CC("SP@");  CC("SP0"); CC("!");

 STR(" .( DEPTH=) DEPTH . CR");  CC("EVAL");

 STR(": INCLUDE-FILE");  CC("EVAL"); // ( fid --- )
 // Read lines from the file identified by fid and interpret them.
 // INCLUDE and EVALUATE nest in arbitrary order.
 STR(" SID @ >R >IN @ >R ATIB @ >R #TIB @ >R RP@ C/L - RP! ");  CC("EVAL");
 STR(" SID ! RP@ ATIB ! ");  CC("EVAL");

 STR(" ['] FQUIT CATCH ");  CC("EVAL");
 STR(" DUP IF CR SOURCE TYPE CR THEN");  CC("EVAL");

 STR(" RP@ C/L + RP! R> #TIB ! R> ATIB ! R> >IN ! R> SID ! THROW ;");  CC("EVAL");

 STR(": INCLUDED");  CC("EVAL"); // ( c-addr u ---- )
 // Open the file with name c-addr u and interpret all lines contained in it.
 STR("R/O OPEN-FILE IF -38 THROW THEN");  CC("EVAL");
 STR("DUP >R ['] INCLUDE-FILE CATCH");  CC("EVAL");
 STR("R> CLOSE-FILE DROP THROW ;");  CC("EVAL");

 STR(": ?ERR. ?DUP IF CR .\" ERR=\" DEC. CR THEN ;");  CC("EVAL");
 STR(": MAIN BEGIN STATE 0! ['] QUIT CATCH ?ERR. SP0 @ SP! AGAIN ;");  CC("EVAL");
 STR(": AUTOEXEC S\" autoexec.4\" ['] INCLUDED CATCH ?ERR. SP0 @ SP! STATE 0! ;"); CC("EVAL");
 STR("AUTOEXEC MAIN"); CC("EVAL");
// S" autoexec.4" INCLUDED
 CC("BYE");
 HERE=shere;

}

/* Free the thread-specific buffer */
 static void buffer_destroy(void * buf)
 {
   free(buf);
 }

 /* Allocate the key */
  static void buffer_key_alloc()
  {
    pthread_key_create(&buffer_key, buffer_destroy);
  }


void *vfm_do(void *pnetid)
{	UNS8 * mem;
	FILE *infile;
	pthread_setspecific(buffer_key, mem=malloc(MEMSIZE+3+4));
	sp=MEMSIZE-2*1024;

	if(*(UNS32*)mainmem) { memcpy(mem,mainmem,MEMSIZE); }
	else	{	make_image();	pmainmem=mem; }

	virtual_machine( *(UNS32 *) pnetid );

	printf("virtual_machine exit\n");
}

void *www_do(void *pnetid)
{	webserver();
}

pthread_attr_t attr;

main(int count, char *strings[])
{ UNS32 ii;
pthread_t pthread;
UNS32 netid;
UNS32 t_ii=0;
UNS32 t_ii0;
UNS8 * mem1;
char dirname[256];

if ( count != 2 )
{
	printf("Usage: %s <portnum>\n", strings[0]);
	exit(0);
}

helpTime=TimeCounter()-360000000;
wwwTime=TimeCounter()-360000000;

NPort=atoi(strings[1]);

	switch( NPort&3)
{
	case 1:	pabdir=pabdir1;	hiddir=hiddir1;	break;
	case 2:	pabdir=pabdir2;	hiddir=hiddir2;	break;
	case 3:	pabdir=pabdir3;	hiddir=hiddir3;	break;
}

	strcpy(HomePath,getenv(HPATH));
	if(strlen(HomePath)==1) strcpy(HomePath,"/root");
	strcpy(dirname,HomePath);
	strcat(dirname,pabdir);

#ifdef WIN32
	mkdir(dirname);
#else
	mkdir(dirname,0777);
#endif

	strcpy(dirname,HomePath);
	strcat(dirname,hiddir);

#ifdef WIN32
	mkdir(dirname);
#else
	mkdir(dirname,0777);

//	signal(SIGINT, SIGINTTypt );
//	signal(SIGALRM, SIGALRMTypt );

	signal(SIGPIPE, SIGSIGPIPETypt);
//	signal(SIGTERM, SIGSIGTERMTypt);
//	signal(SIGSEGV, SIGSIGSEGVTypt);

#endif
//portnum = strings[1];

int sss=c_serv(NPort);

// mem1=malloc(1000); free(mem1);

   mainmem=malloc(MEMSIZE+3+4);
   *(UNS32*)mainmem=0;

   pthread_attr_init(&attr);
   pthread_once(&buffer_key_once, buffer_key_alloc);
   pthread_mutex_init(&mutexacc, NULL);

 netid=0;
 f_struct[t_ii].netid=netid;
 f_struct[t_ii].busy=1;
 f_struct[t_ii].fpos=-1;
   pthread_create(&pthread, &attr, vfm_do, (void *)&t_ii);
//  while(*(volatile UNS32*)mainmem==0);
  while(stage==0);

  { int  dirsize= strlen(dirname);

  strcat(dirname,"/f.log");
  netid=open( dirname , O_RDWR|O_CREAT|O_APPEND|_O_BINARY , S_IRUSR|S_IWUSR );

  dirname[dirsize]=0;
  strcat(dirname,"/acc.dat");
  accdat=open( dirname , O_RDWR|O_CREAT|O_APPEND|_O_BINARY , S_IRUSR|S_IWUSR );
  close(accdat);
  accdat=open( dirname , O_RDWR|_O_BINARY , S_IRUSR|S_IWUSR );

  dirname[dirsize]=0;
  strcat(dirname,"/hosts.dat");
  HostsDat=open( dirname , O_RDWR|O_CREAT|O_APPEND|_O_BINARY , S_IRUSR|S_IWUSR );
  close(HostsDat);
  HostsDat=open( dirname , O_RDWR|_O_BINARY , S_IRUSR|S_IWUSR );

  dirname[dirsize]=0;
  strcat(dirname,"/sets.4");
  {  UNS32  setFl;
  setFl=open( dirname , O_RDWR|O_CREAT|O_APPEND|_O_BINARY , S_IRUSR|S_IWUSR );
  close(setFl);
  }

  dirname[dirsize]=0;
  strcat(dirname,"/help.txt");
  { UNS32  HelpFl;
    HelpFl=open( dirname , O_RDWR|O_CREAT|O_APPEND|_O_BINARY , S_IRUSR|S_IWUSR );
    if(lseek(HelpFl, (off_t) 0, SEEK_END)==0)
    { UNS32  HelpFlLoc, res;	char fbuf[300]; 

	HelpFlLoc=open("help.txt",O_RDONLY|_O_BINARY,S_IRUSR | S_IWUSR );
		while( !eof(HelpFlLoc) )
		{ res=read(HelpFlLoc,&fbuf,300);
		  if(res<=300)
		  write(HelpFl,fbuf,res);
		}
	close(HelpFlLoc);
    }	close(HelpFl);
   }
  }

  read(accdat, (char *) &f_struct[0].data , sizeof(f_data_t));


  f_struct[++t_ii].netid=netid;
  f_struct[t_ii].busy=1;
  f_struct[t_ii].fpos=-1;

  pthread_create(&pthread, &attr, vfm_do, (void *)&t_ii);

  for(ii=t_ii+1;ii<f_structMax;ii++)
	{ f_struct[ii].busy=0; f_struct[ii].fpos=-1; }

 pthread_create(&pthread, &attr, vfm_do, (void *)&t_ii);

 pthread_create(&pthread, &attr, www_do, (void *)0);

  while(stage==1);

  stage=20;
  t_ii0=t_ii;
  for(;;)
 {
	while(f_struct[t_ii0].busy)
		{ t_ii0=f_structMax&(t_ii0+1); usleep(9);
		}
 f_struct[t_ii0].netid=c_accept(sss)|0x80000000;
 f_struct[t_ii0].busy=1;
 f_struct[t_ii0].fpos=-1;
 f_struct[t_ii0].pp=0;
 f_struct[t_ii0].accountSpent=0L;
 f_struct[t_ii0].accountRecd=0L;
 f_struct[t_ii0].barterSpent=0L;
 f_struct[t_ii0].CIPAddr=CIPAddr;
 f_struct[t_ii0].data.account=0L;
 f_struct[t_ii0].data.barter=0L;

// for(ii=0;ii<=t_ii0;ii++)  printf(" ii= %d %d %d\n", ii, f_struct[ii].busy,f_struct[ii].fpos);


 t_ii=t_ii0;
 pthread_create(&pthread, &attr, vfm_do, (void *)&t_ii);
 t_ii0++;
//  for(;;);
 }
  pthread_mutex_destroy(&mutexacc);
 return 0;
}

